@@ -2,25 +2,31 @@
 
 #include "StdAfx.h"
 
-#include "Common/StringConvert.h"
-#include "Common/Defs.h"
-#include "Common/UTFConvert.h"
+#include "../../../../C/Alloc.h"
+
+#include "Common/Buffer.h"
 #include "Common/ComTry.h"
+#include "Common/Defs.h"
 #include "Common/IntToString.h"
+#include "Common/StringConvert.h"
+#include "Common/UTFConvert.h"
 
 #include "Windows/PropVariant.h"
 #include "Windows/Time.h"
 
-#include "CabHandler.h"
-#include "CabBlockInStream.h"
+#include "../../Common/ProgressUtils.h"
+#include "../../Common/StreamUtils.h"
 
-#include "../../Compress/Copy/CopyCoder.h"
-#include "../../Compress/Deflate/DeflateDecoder.h"
-#include "../../Compress/Lzx/LzxDecoder.h"
-#include "../../Compress/Quantum/QuantumDecoder.h"
+#include "../../Compress/CopyCoder.h"
+#include "../../Compress/DeflateDecoder.h"
+#include "../../Compress/LzxDecoder.h"
+#include "../../Compress/QuantumDecoder.h"
 
 #include "../Common/ItemNameUtils.h"
 
+#include "CabBlockInStream.h"
+#include "CabHandler.h"
+
 using namespace NWindows;
 
 namespace NArchive {
@@ -29,34 +35,29 @@ namespace NCab {
 // #define _CAB_DETAILS
 
 #ifdef _CAB_DETAILS
-enum 
+enum
 {
-  kpidBlockReal = kpidUserDefined,
-  kpidOffset,
-  kpidVolume,
+  kpidBlockReal = kpidUserDefined
 };
 #endif
 
-STATPROPSTG kProperties[] = 
+STATPROPSTG kProps[] =
 {
   { NULL, kpidPath, VT_BSTR},
-  // { NULL, kpidIsFolder, VT_BOOL},
   { NULL, kpidSize, VT_UI8},
-  { NULL, kpidLastWriteTime, VT_FILETIME},
-  { NULL, kpidAttributes, VT_UI4},
+  { NULL, kpidMTime, VT_FILETIME},
+  { NULL, kpidAttrib, VT_UI4},
   { NULL, kpidMethod, VT_BSTR},
   { NULL, kpidBlock, VT_I4}
   #ifdef _CAB_DETAILS
   ,
   { L"BlockReal", kpidBlockReal, VT_UI4},
-  { L"Offset", kpidOffset, VT_UI4},
-  { L"Volume", kpidVolume, VT_UI4}
+  { NULL, kpidOffset, VT_UI4},
+  { NULL, kpidVolume, VT_UI4}
   #endif
 };
 
-static const int kNumProperties = sizeof(kProperties) / sizeof(kProperties[0]);
-
-static const wchar_t *kMethods[] = 
+static const wchar_t *kMethods[] =
 {
   L"None",
   L"MSZip",
@@ -67,49 +68,69 @@ static const wchar_t *kMethods[] =
 static const int kNumMethods = sizeof(kMethods) / sizeof(kMethods[0]);
 static const wchar_t *kUnknownMethod = L"Unknown";
 
-STDMETHODIMP CHandler::GetArchiveProperty(PROPID /* propID */, PROPVARIANT *value)
+STATPROPSTG kArcProps[] =
 {
-  value->vt = VT_EMPTY;
-  return S_OK;
-}
+  { NULL, kpidMethod, VT_BSTR},
+  // { NULL, kpidSolid, VT_BOOL},
+  { NULL, kpidNumBlocks, VT_UI4},
+  { NULL, kpidNumVolumes, VT_UI4}
+};
 
-STDMETHODIMP CHandler::GetNumberOfProperties(UInt32 *numProperties)
-{
-  *numProperties = sizeof(kProperties) / sizeof(kProperties[0]);
-  return S_OK;
-}
+IMP_IInArchive_Props
+IMP_IInArchive_ArcProps
 
-STDMETHODIMP CHandler::GetPropertyInfo(UInt32 index,     
-      BSTR *name, PROPID *propID, VARTYPE *varType)
+STDMETHODIMP CHandler::GetArchiveProperty(PROPID propID, PROPVARIANT *value)
 {
-  if(index >= sizeof(kProperties) / sizeof(kProperties[0]))
-    return E_INVALIDARG;
-  const STATPROPSTG &srcItem = kProperties[index];
-  *propID = srcItem.propid;
-  *varType = srcItem.vt;
-  if (srcItem.lpwstrName == 0)
-    *name = 0;
-  else
-    *name = ::SysAllocString(srcItem.lpwstrName);
+  COM_TRY_BEGIN
+  NWindows::NCOM::CPropVariant prop;
+  switch(propID)
+  {
+    case kpidMethod:
+    {
+      UString resString;
+      CRecordVector<Byte> ids;
+      int i;
+      for (int v = 0; v < m_Database.Volumes.Size(); v++)
+      {
+        const CDatabaseEx &de = m_Database.Volumes[v];
+        for (i = 0; i < de.Folders.Size(); i++)
+          ids.AddToUniqueSorted(de.Folders[i].GetCompressionMethod());
+      }
+      for (i = 0; i < ids.Size(); i++)
+      {
+        Byte id = ids[i];
+        UString method = (id < kNumMethods) ? kMethods[id] : kUnknownMethod;
+        if (!resString.IsEmpty())
+          resString += L' ';
+        resString += method;
+      }
+      prop = resString;
+      break;
+    }
+    // case kpidSolid: prop = _database.IsSolid(); break;
+    case kpidNumBlocks:
+    {
+      UInt32 numFolders = 0;
+      for (int v = 0; v < m_Database.Volumes.Size(); v++)
+        numFolders += m_Database.Volumes[v].Folders.Size();
+      prop = numFolders;
+      break;
+    }
+    case kpidNumVolumes:
+    {
+      prop = (UInt32)m_Database.Volumes.Size();
+      break;
+    }
+  }
+  prop.Detach(value);
   return S_OK;
-}
-
-STDMETHODIMP CHandler::GetNumberOfArchiveProperties(UInt32 *numProperties)
-{
-  *numProperties = 0;
-  return S_OK;
-}
-
-STDMETHODIMP CHandler::GetArchivePropertyInfo(UInt32 /* index */,     
-      BSTR * /* name */, PROPID * /* propID */, VARTYPE * /* varType */)
-{
-  return E_INVALIDARG;
+  COM_TRY_END
 }
 
 STDMETHODIMP CHandler::GetProperty(UInt32 index, PROPID propID,  PROPVARIANT *value)
 {
   COM_TRY_BEGIN
-  NWindows::NCOM::CPropVariant propVariant;
+  NWindows::NCOM::CPropVariant prop;
   
   const CMvItem &mvItem = m_Database.Items[index];
   const CDatabaseEx &db = m_Database.Volumes[mvItem.VolumeIndex];
@@ -124,16 +145,14 @@ STDMETHODIMP CHandler::GetProperty(UInt32 index, PROPID propID,  PROPVARIANT *va
         ConvertUTF8ToUnicode(item.Name, unicodeName);
       else
         unicodeName = MultiByteToUnicodeString(item.Name, CP_ACP);
-      propVariant = (const wchar_t *)NItemName::WinNameToOSName(unicodeName);
+      prop = (const wchar_t *)NItemName::WinNameToOSName(unicodeName);
       break;
     }
-    case kpidIsFolder:
-      propVariant = item.IsDirectory();
-      break;
-    case kpidSize:
-      propVariant = item.Size;
-      break;
-    case kpidLastWriteTime:
+    case kpidIsDir:  prop = item.IsDir(); break;
+    case kpidSize:  prop = item.Size; break;
+    case kpidAttrib:  prop = item.GetWinAttributes(); break;
+
+    case kpidMTime:
     {
       FILETIME localFileTime, utcFileTime;
       if (NTime::DosTimeToFileTime(item.Time, localFileTime))
@@ -143,12 +162,9 @@ STDMETHODIMP CHandler::GetProperty(UInt32 index, PROPID propID,  PROPVARIANT *va
       }
       else
         utcFileTime.dwHighDateTime = utcFileTime.dwLowDateTime = 0;
-      propVariant = utcFileTime;
+      prop = utcFileTime;
       break;
     }
-    case kpidAttributes:
-      propVariant = item.GetWinAttributes();
-      break;
 
     case kpidMethod:
     {
@@ -156,7 +172,7 @@ STDMETHODIMP CHandler::GetProperty(UInt32 index, PROPID propID,  PROPVARIANT *va
       const CFolder &folder = db.Folders[realFolderIndex];
       int methodIndex = folder.GetCompressionMethod();
       UString method = (methodIndex < kNumMethods) ? kMethods[methodIndex] : kUnknownMethod;
-      if (methodIndex == NHeader::NCompressionMethodMajor::kLZX || 
+      if (methodIndex == NHeader::NCompressionMethodMajor::kLZX ||
         methodIndex == NHeader::NCompressionMethodMajor::kQuantum)
       {
         method += L":";
@@ -164,34 +180,26 @@ STDMETHODIMP CHandler::GetProperty(UInt32 index, PROPID propID,  PROPVARIANT *va
         ConvertUInt64ToString(folder.CompressionTypeMinor, temp);
         method += temp;
       }
-      propVariant = method;
+      prop = method;
       break;
     }
-    case kpidBlock:
-      propVariant = (Int32)m_Database.GetFolderIndex(&mvItem);
-      break;
+    case kpidBlock:  prop = (Int32)m_Database.GetFolderIndex(&mvItem); break;
     
     #ifdef _CAB_DETAILS
     
-    case kpidBlockReal:
-      propVariant = UInt32(item.FolderIndex);
-      break;
-    case kpidOffset:
-      propVariant = (UInt32)item.Offset;
-      break;
-    case kpidVolume:
-      propVariant = (UInt32)mvItem.VolumeIndex;
-      break;
-    
+    case kpidBlockReal:  prop = (UInt32)item.FolderIndex; break;
+    case kpidOffset:  prop = (UInt32)item.Offset; break;
+    case kpidVolume:  prop = (UInt32)mvItem.VolumeIndex; break;
+
     #endif
   }
-  propVariant.Detach(value);
+  prop.Detach(value);
   return S_OK;
   COM_TRY_END
 }
 
 /*
-class CPropgressImp: public CProgressVirt
+class CProgressImp: public CProgressVirt
 {
   CMyComPtr<IArchiveOpenCallback> m_OpenArchiveCallback;
 public:
@@ -201,14 +209,14 @@ public:
     { m_OpenArchiveCallback = openArchiveCallback; }
 };
 
-STDMETHODIMP CPropgressImp::SetTotal(const UInt64 *numFiles)
+STDMETHODIMP CProgressImp::SetTotal(const UInt64 *numFiles)
 {
   if (m_OpenArchiveCallback)
     return m_OpenArchiveCallback->SetCompleted(numFiles, NULL);
   return S_OK;
 }
 
-STDMETHODIMP CPropgressImp::SetCompleted(const UInt64 *numFiles)
+STDMETHODIMP CProgressImp::SetCompleted(const UInt64 *numFiles)
 {
   if (m_OpenArchiveCallback)
     return m_OpenArchiveCallback->SetCompleted(numFiles, NULL);
@@ -216,26 +224,23 @@ STDMETHODIMP CPropgressImp::SetCompleted(const UInt64 *numFiles)
 }
 */
 
-STDMETHODIMP CHandler::Open(IInStream *inStream, 
+STDMETHODIMP CHandler::Open(IInStream *inStream,
     const UInt64 *maxCheckStartPosition,
-    IArchiveOpenCallback *openArchiveCallback)
+    IArchiveOpenCallback *callback)
 {
   COM_TRY_BEGIN
   Close();
   HRESULT res = S_FALSE;
   CInArchive archive;
   CMyComPtr<IArchiveOpenVolumeCallback> openVolumeCallback;
-  {
-    CMyComPtr<IArchiveOpenCallback> openArchiveCallbackWrap = openArchiveCallback;
-    openArchiveCallbackWrap.QueryInterface(IID_IArchiveOpenVolumeCallback, &openVolumeCallback);
-  }
-
+  callback->QueryInterface(IID_IArchiveOpenVolumeCallback, (void **)&openVolumeCallback);
+  
   CMyComPtr<IInStream> nextStream = inStream;
   bool prevChecked = false;
   UInt64 numItems = 0;
   try
   {
-    while(nextStream != 0)
+    while (nextStream != 0)
     {
       CDatabaseEx db;
       db.Stream = nextStream;
@@ -246,7 +251,7 @@ STDMETHODIMP CHandler::Open(IInStream *inStream,
         {
           const CDatabaseEx &dbPrev = m_Database.Volumes[prevChecked ? m_Database.Volumes.Size() - 1 : 0];
           if (dbPrev.ArchiveInfo.SetID != db.ArchiveInfo.SetID ||
-              dbPrev.ArchiveInfo.CabinetNumber + (prevChecked ? 1: - 1) != 
+              dbPrev.ArchiveInfo.CabinetNumber + (prevChecked ? 1: - 1) !=
               db.ArchiveInfo.CabinetNumber)
             res = S_FALSE;
         }
@@ -255,7 +260,7 @@ STDMETHODIMP CHandler::Open(IInStream *inStream,
         m_Database.Volumes.Insert(prevChecked ? m_Database.Volumes.Size() : 0, db);
       else if (res != S_FALSE)
         return res;
-      else 
+      else
       {
         if (m_Database.Volumes.IsEmpty())
           return S_FALSE;
@@ -265,7 +270,7 @@ STDMETHODIMP CHandler::Open(IInStream *inStream,
       }
 
       numItems += db.Items.Size();
-      RINOK(openArchiveCallback->SetCompleted(&numItems, NULL));
+      RINOK(callback->SetCompleted(&numItems, NULL));
         
       nextStream = 0;
       for (;;)
@@ -275,7 +280,7 @@ STDMETHODIMP CHandler::Open(IInStream *inStream,
         {
           const CInArchiveInfo &ai = m_Database.Volumes.Front().ArchiveInfo;
           if (ai.IsTherePrev())
-            otherArchive = &ai.PreviousArchive;
+            otherArchive = &ai.PrevArc;
           else
             prevChecked = true;
         }
@@ -283,11 +288,14 @@ STDMETHODIMP CHandler::Open(IInStream *inStream,
         {
           const CInArchiveInfo &ai = m_Database.Volumes.Back().ArchiveInfo;
           if (ai.IsThereNext())
-            otherArchive = &ai.NextArchive;
+            otherArchive = &ai.NextArc;
         }
         if (!otherArchive)
           break;
         const UString fullName = MultiByteToUnicodeString(otherArchive->FileName, CP_ACP);
+        if (!openVolumeCallback)
+          break;
+
         HRESULT result = openVolumeCallback->GetStream(fullName, &nextStream);
         if (result == S_OK)
           break;
@@ -324,7 +332,7 @@ STDMETHODIMP CHandler::Close()
   return S_OK;
 }
 
-class CCabFolderOutStream: 
+class CFolderOutStream:
   public ISequentialOutStream,
   public CMyUnknownImp
 {
@@ -335,6 +343,13 @@ public:
 private:
   const CMvDatabaseEx *m_Database;
   const CRecordVector<bool> *m_ExtractStatuses;
+  
+  Byte *TempBuf;
+  UInt32 TempBufSize;
+  int NumIdenticalFiles;
+  bool TempBufMode;
+  UInt32 m_BufStartFolderOffset;
+
   int m_StartIndex;
   int m_CurrentIndex;
   CMyComPtr<IArchiveExtractCallback> m_ExtractCallback;
@@ -344,19 +359,29 @@ private:
 
   bool m_IsOk;
   bool m_FileIsOpen;
-  UInt64 m_RemainFileSize;
+  UInt32 m_RemainFileSize;
   UInt64 m_FolderSize;
   UInt64 m_PosInFolder;
 
+  void FreeTempBuf()
+  {
+    ::MyFree(TempBuf);
+    TempBuf = NULL;
+  }
+
   HRESULT OpenFile();
+  HRESULT CloseFileWithResOp(Int32 resOp);
+  HRESULT CloseFile();
   HRESULT Write2(const void *data, UInt32 size, UInt32 *processedSize, bool isOK);
 public:
   HRESULT WriteEmptyFiles();
 
+  CFolderOutStream(): TempBuf(NULL) {}
+  ~CFolderOutStream() { FreeTempBuf(); }
   void Init(
       const CMvDatabaseEx *database,
-      const CRecordVector<bool> *extractStatuses, 
-      int startIndex, 
+      const CRecordVector<bool> *extractStatuses,
+      int startIndex,
       UInt64 folderSize,
       IArchiveExtractCallback *extractCallback,
       bool testMode);
@@ -367,10 +392,10 @@ public:
   UInt64 GetPosInFolder() const { return m_PosInFolder; }
 };
 
-void CCabFolderOutStream::Init(
+void CFolderOutStream::Init(
     const CMvDatabaseEx *database,
-    const CRecordVector<bool> *extractStatuses, 
-    int startIndex, 
+    const CRecordVector<bool> *extractStatuses,
+    int startIndex,
     UInt64 folderSize,
     IArchiveExtractCallback *extractCallback,
     bool testMode)
@@ -387,25 +412,92 @@ void CCabFolderOutStream::Init(
   m_PosInFolder = 0;
   m_FileIsOpen = false;
   m_IsOk = true;
+  TempBufMode = false;
+  NumIdenticalFiles = 0;
 }
 
-HRESULT CCabFolderOutStream::OpenFile()
+HRESULT CFolderOutStream::CloseFileWithResOp(Int32 resOp)
 {
-  Int32 askMode = (*m_ExtractStatuses)[m_CurrentIndex] ? (m_TestMode ? 
+  m_RealOutStream.Release();
+  m_FileIsOpen = false;
+  NumIdenticalFiles--;
+  return m_ExtractCallback->SetOperationResult(resOp);
+}
+
+HRESULT CFolderOutStream::CloseFile()
+{
+  return CloseFileWithResOp(m_IsOk ?
+      NExtract::NOperationResult::kOK:
+      NExtract::NOperationResult::kDataError);
+}
+
+HRESULT CFolderOutStream::OpenFile()
+{
+  if (NumIdenticalFiles == 0)
+  {
+    const CMvItem &mvItem = m_Database->Items[m_StartIndex + m_CurrentIndex];
+    const CItem &item = m_Database->Volumes[mvItem.VolumeIndex].Items[mvItem.ItemIndex];
+    int numExtractItems = 0;
+    int curIndex;
+    for (curIndex = m_CurrentIndex; curIndex < m_ExtractStatuses->Size(); curIndex++)
+    {
+      const CMvItem &mvItem2 = m_Database->Items[m_StartIndex + curIndex];
+      const CItem &item2 = m_Database->Volumes[mvItem2.VolumeIndex].Items[mvItem2.ItemIndex];
+      if (item.Offset != item2.Offset ||
+          item.Size != item2.Size ||
+          item.Size == 0)
+        break;
+      if (!m_TestMode && (*m_ExtractStatuses)[curIndex])
+        numExtractItems++;
+    }
+    NumIdenticalFiles = (curIndex - m_CurrentIndex);
+    if (NumIdenticalFiles == 0)
+      NumIdenticalFiles = 1;
+    TempBufMode = false;
+    if (numExtractItems > 1)
+    {
+      if (!TempBuf || item.Size > TempBufSize)
+      {
+        FreeTempBuf();
+        TempBuf = (Byte *)MyAlloc(item.Size);
+        TempBufSize = item.Size;
+        if (TempBuf == NULL)
+          return E_OUTOFMEMORY;
+      }
+      TempBufMode = true;
+      m_BufStartFolderOffset = item.Offset;
+    }
+    else if (numExtractItems == 1)
+    {
+      while (NumIdenticalFiles && !(*m_ExtractStatuses)[m_CurrentIndex])
+      {
+        CMyComPtr<ISequentialOutStream> stream;
+        RINOK(m_ExtractCallback->GetStream(m_StartIndex + m_CurrentIndex, &stream, NExtract::NAskMode::kSkip));
+        if (stream)
+          return E_FAIL;
+        RINOK(m_ExtractCallback->PrepareOperation(NExtract::NAskMode::kSkip));
+        m_CurrentIndex++;
+        m_FileIsOpen = true;
+        CloseFile();
+      }
+    }
+  }
+
+  Int32 askMode = (*m_ExtractStatuses)[m_CurrentIndex] ? (m_TestMode ?
       NExtract::NAskMode::kTest :
       NExtract::NAskMode::kExtract) :
       NExtract::NAskMode::kSkip;
   RINOK(m_ExtractCallback->GetStream(m_StartIndex + m_CurrentIndex, &m_RealOutStream, askMode));
   if (!m_RealOutStream && !m_TestMode)
-    askMode = NArchive::NExtract::NAskMode::kSkip;
+    askMode = NExtract::NAskMode::kSkip;
   return m_ExtractCallback->PrepareOperation(askMode);
 }
 
-HRESULT CCabFolderOutStream::WriteEmptyFiles()
+HRESULT CFolderOutStream::WriteEmptyFiles()
 {
   if (m_FileIsOpen)
     return S_OK;
-  for(;m_CurrentIndex < m_ExtractStatuses->Size(); m_CurrentIndex++)
+  for (; m_CurrentIndex < m_ExtractStatuses->Size(); m_CurrentIndex++)
   {
     const CMvItem &mvItem = m_Database->Items[m_StartIndex + m_CurrentIndex];
     const CItem &item = m_Database->Volumes[mvItem.VolumeIndex].Items[mvItem.ItemIndex];
@@ -415,22 +507,23 @@ HRESULT CCabFolderOutStream::WriteEmptyFiles()
     HRESULT result = OpenFile();
     m_RealOutStream.Release();
     RINOK(result);
-    RINOK(m_ExtractCallback->SetOperationResult(NArchive::NExtract::NOperationResult::kOK));
+    RINOK(m_ExtractCallback->SetOperationResult(NExtract::NOperationResult::kOK));
   }
   return S_OK;
 }
 
 // This is Write function
-HRESULT CCabFolderOutStream::Write2(const void *data, UInt32 size, UInt32 *processedSize, bool isOK)
+HRESULT CFolderOutStream::Write2(const void *data, UInt32 size, UInt32 *processedSize, bool isOK)
 {
+  COM_TRY_BEGIN
   UInt32 realProcessed = 0;
   if (processedSize != NULL)
    *processedSize = 0;
-  while(size != 0)
+  while (size != 0)
   {
     if (m_FileIsOpen)
     {
-      UInt32 numBytesToWrite = (UInt32)MyMin(m_RemainFileSize, (UInt64)(size));
+      UInt32 numBytesToWrite = MyMin(m_RemainFileSize, size);
       HRESULT res = S_OK;
       if (numBytesToWrite > 0)
       {
@@ -442,6 +535,8 @@ HRESULT CCabFolderOutStream::Write2(const void *data, UInt32 size, UInt32 *proce
           res = m_RealOutStream->Write((const Byte *)data, numBytesToWrite, &processedSizeLocal);
           numBytesToWrite = processedSizeLocal;
         }
+        if (TempBufMode && TempBuf)
+          memcpy(TempBuf + (m_PosInFolder - m_BufStartFolderOffset), data, numBytesToWrite);
       }
       realProcessed += numBytesToWrite;
       if (processedSize != NULL)
@@ -454,12 +549,27 @@ HRESULT CCabFolderOutStream::Write2(const void *data, UInt32 size, UInt32 *proce
         return res;
       if (m_RemainFileSize == 0)
       {
-        m_RealOutStream.Release();
-        RINOK(m_ExtractCallback->SetOperationResult(
-          m_IsOk ? 
-            NArchive::NExtract::NOperationResult::kOK:
-            NArchive::NExtract::NOperationResult::kDataError));
-        m_FileIsOpen = false;
+        RINOK(CloseFile());
+
+        while (NumIdenticalFiles)
+        {
+          HRESULT result = OpenFile();
+          m_FileIsOpen = true;
+          m_CurrentIndex++;
+          if (result == S_OK && m_RealOutStream && TempBuf)
+            result = WriteStream(m_RealOutStream, TempBuf, (size_t)(m_PosInFolder - m_BufStartFolderOffset));
+          
+          if (!TempBuf && TempBufMode && m_RealOutStream)
+          {
+            RINOK(CloseFileWithResOp(NExtract::NOperationResult::kUnSupportedMethod));
+          }
+          else
+          {
+            RINOK(CloseFile());
+          }
+          RINOK(result);
+        }
+        TempBufMode = false;
       }
       if (realProcessed > 0)
         break; // with this break this function works as Write-Part
@@ -479,7 +589,7 @@ HRESULT CCabFolderOutStream::Write2(const void *data, UInt32 size, UInt32 *proce
         return E_FAIL;
       if (fileOffset > m_PosInFolder)
       {
-        UInt32 numBytesToWrite = (UInt32)MyMin((UInt64)fileOffset - m_PosInFolder, UInt64(size));
+        UInt32 numBytesToWrite = MyMin(fileOffset - (UInt32)m_PosInFolder, size);
         realProcessed += numBytesToWrite;
         if (processedSize != NULL)
           *processedSize = realProcessed;
@@ -497,14 +607,15 @@ HRESULT CCabFolderOutStream::Write2(const void *data, UInt32 size, UInt32 *proce
     }
   }
   return WriteEmptyFiles();
+  COM_TRY_END
 }
 
-STDMETHODIMP CCabFolderOutStream::Write(const void *data, UInt32 size, UInt32 *processedSize)
+STDMETHODIMP CFolderOutStream::Write(const void *data, UInt32 size, UInt32 *processedSize)
 {
   return Write2(data, size, processedSize, true);
 }
 
-HRESULT CCabFolderOutStream::FlushCorrupted()
+HRESULT CFolderOutStream::FlushCorrupted()
 {
   const UInt32 kBufferSize = (1 << 10);
   Byte buffer[kBufferSize];
@@ -521,7 +632,7 @@ HRESULT CCabFolderOutStream::FlushCorrupted()
   }
 }
 
-HRESULT CCabFolderOutStream::Unsupported()
+HRESULT CFolderOutStream::Unsupported()
 {
   while(m_CurrentIndex < m_ExtractStatuses->Size())
   {
@@ -529,23 +640,23 @@ HRESULT CCabFolderOutStream::Unsupported()
     if (result != S_FALSE && result != S_OK)
       return result;
     m_RealOutStream.Release();
-    RINOK(m_ExtractCallback->SetOperationResult(NArchive::NExtract::NOperationResult::kUnSupportedMethod));
+    RINOK(m_ExtractCallback->SetOperationResult(NExtract::NOperationResult::kUnSupportedMethod));
     m_CurrentIndex++;
   }
   return S_OK;
 }
 
 
-STDMETHODIMP CHandler::Extract(const UInt32* indices, UInt32 numItems,
-    Int32 _aTestMode, IArchiveExtractCallback *extractCallback)
+STDMETHODIMP CHandler::Extract(const UInt32 *indices, UInt32 numItems,
+    Int32 testModeSpec, IArchiveExtractCallback *extractCallback)
 {
   COM_TRY_BEGIN
-  bool allFilesMode = (numItems == (UInt32)(-1));
+  bool allFilesMode = (numItems == (UInt32)-1);
   if (allFilesMode)
     numItems = m_Database.Items.Size();
   if(numItems == 0)
     return S_OK;
-  bool testMode = (_aTestMode != 0);
+  bool testMode = (testModeSpec != 0);
   UInt64 totalUnPacked = 0;
 
   UInt32 i;
@@ -556,7 +667,7 @@ STDMETHODIMP CHandler::Extract(const UInt32* indices, UInt32 numItems,
     int index = allFilesMode ? i : indices[i];
     const CMvItem &mvItem = m_Database.Items[index];
     const CItem &item = m_Database.Volumes[mvItem.VolumeIndex].Items[mvItem.ItemIndex];
-    if (item.IsDirectory())
+    if (item.IsDir())
       continue;
     int folderIndex = m_Database.GetFolderIndex(&mvItem);
     if (folderIndex != lastFolder)
@@ -570,8 +681,14 @@ STDMETHODIMP CHandler::Extract(const UInt32* indices, UInt32 numItems,
 
   totalUnPacked = 0;
 
-  NCompress::CCopyCoder *copyCoderSpec = NULL;
-  CMyComPtr<ICompressCoder> copyCoder;
+  UInt64 totalPacked = 0;
+
+  CLocalProgress *lps = new CLocalProgress;
+  CMyComPtr<ICompressProgressInfo> progress = lps;
+  lps->Init(extractCallback, false);
+
+  NCompress::CCopyCoder *copyCoderSpec = new NCompress::CCopyCoder;
+  CMyComPtr<ICompressCoder> copyCoder = copyCoderSpec;
 
   NCompress::NDeflate::NDecoder::CCOMCoder *deflateDecoderSpec = NULL;
   CMyComPtr<ICompressCoder> deflateDecoder;
@@ -598,30 +715,30 @@ STDMETHODIMP CHandler::Extract(const UInt32* indices, UInt32 numItems,
     const CItem &item = db.Items[itemIndex];
 
     i++;
-    if (item.IsDirectory())
+    if (item.IsDir())
     {
-      Int32 askMode= testMode ? 
-          NArchive::NExtract::NAskMode::kTest :
-          NArchive::NExtract::NAskMode::kExtract;
+      Int32 askMode = testMode ?
+          NExtract::NAskMode::kTest :
+          NExtract::NAskMode::kExtract;
       CMyComPtr<ISequentialOutStream> realOutStream;
       RINOK(extractCallback->GetStream(index, &realOutStream, askMode));
       RINOK(extractCallback->PrepareOperation(askMode));
       realOutStream.Release();
-      RINOK(extractCallback->SetOperationResult(NArchive::NExtract::NOperationResult::kOK));
+      RINOK(extractCallback->SetOperationResult(NExtract::NOperationResult::kOK));
       continue;
     }
     int folderIndex = m_Database.GetFolderIndex(&mvItem);
     if (folderIndex < 0)
     {
       // If we need previous archive
-      Int32 askMode= testMode ? 
-          NArchive::NExtract::NAskMode::kTest :
-          NArchive::NExtract::NAskMode::kExtract;
+      Int32 askMode= testMode ?
+          NExtract::NAskMode::kTest :
+          NExtract::NAskMode::kExtract;
       CMyComPtr<ISequentialOutStream> realOutStream;
       RINOK(extractCallback->GetStream(index, &realOutStream, askMode));
       RINOK(extractCallback->PrepareOperation(askMode));
       realOutStream.Release();
-      RINOK(extractCallback->SetOperationResult(NArchive::NExtract::NOperationResult::kDataError));
+      RINOK(extractCallback->SetOperationResult(NExtract::NOperationResult::kDataError));
       continue;
     }
     int startIndex2 = m_Database.FolderStartFileIndex[folderIndex];
@@ -637,7 +754,7 @@ STDMETHODIMP CHandler::Extract(const UInt32* indices, UInt32 numItems,
       int indexNext = allFilesMode ? i : indices[i];
       const CMvItem &mvItem = m_Database.Items[indexNext];
       const CItem &item = m_Database.Volumes[mvItem.VolumeIndex].Items[mvItem.ItemIndex];
-      if (item.IsDirectory())
+      if (item.IsDir())
         continue;
       int newFolderIndex = m_Database.GetFolderIndex(&mvItem);
 
@@ -650,25 +767,22 @@ STDMETHODIMP CHandler::Extract(const UInt32* indices, UInt32 numItems,
       curUnpack = item.GetEndOffset();
     }
 
-    RINOK(extractCallback->SetCompleted(&totalUnPacked));
-    
-    CCabFolderOutStream *cabFolderOutStream = new CCabFolderOutStream;
+    lps->OutSize = totalUnPacked;
+    lps->InSize = totalPacked;
+    RINOK(lps->SetCur());
+
+    CFolderOutStream *cabFolderOutStream = new CFolderOutStream;
     CMyComPtr<ISequentialOutStream> outStream(cabFolderOutStream);
 
     const CFolder &folder = db.Folders[item.GetFolderIndex(db.Folders.Size())];
 
-    cabFolderOutStream->Init(&m_Database, &extractStatuses, startIndex2, 
+    cabFolderOutStream->Init(&m_Database, &extractStatuses, startIndex2,
         curUnpack, extractCallback, testMode);
 
     cabBlockInStreamSpec->MsZip = false;
     switch(folder.GetCompressionMethod())
     {
       case NHeader::NCompressionMethodMajor::kNone:
-        if(copyCoderSpec == NULL)
-        {
-          copyCoderSpec = new NCompress::CCopyCoder;
-          copyCoder = copyCoderSpec;
-        }
         break;
       case NHeader::NCompressionMethodMajor::kMSZip:
         if(deflateDecoderSpec == NULL)
@@ -750,9 +864,13 @@ STDMETHODIMP CHandler::Extract(const UInt32* indices, UInt32 numItems,
         if (keepInputBuffer)
           continue;
 
-
         UInt64 totalUnPacked2 = totalUnPacked + cabFolderOutStream->GetPosInFolder();
-        RINOK(extractCallback->SetCompleted(&totalUnPacked2));
+        totalPacked += packSize;
+
+        lps->OutSize = totalUnPacked2;
+        lps->InSize = totalPacked;
+        RINOK(lps->SetCur());
+
         UInt64 unpackRemain = cabFolderOutStream->GetRemain();
 
         const UInt32 kBlockSizeMax = (1 << 15);
@@ -771,7 +889,7 @@ STDMETHODIMP CHandler::Extract(const UInt32* indices, UInt32 numItems,
             res = deflateDecoder->Code(cabBlockInStream, outStream, NULL, &unpackRemain, NULL);
             break;
           case NHeader::NCompressionMethodMajor::kLZX:
-            lzxDecoderSpec->SetKeepHistory(keepHistory, cabBlockInStreamSpec->GetAlign());
+            lzxDecoderSpec->SetKeepHistory(keepHistory);
             res = lzxDecoder->Code(cabBlockInStream, outStream, NULL, &unpackRemain, NULL);
             break;
           case NHeader::NCompressionMethodMajor::kQuantum:
