@@ -2,19 +2,19 @@
 
 #include "StdAfx.h"
 
-#include "NsisHandler.h"
+#include "../../../../C/CpuArch.h"
 
-#include "Common/StringConvert.h"
+#include "Common/ComTry.h"
 #include "Common/IntToString.h"
 #include "Common/NewHandler.h"
-#include "Common/ComTry.h"
 
 #include "Windows/PropVariant.h"
 
-#include "../Common/ItemNameUtils.h"
 #include "../../Common/StreamUtils.h"
 
-#include "../../../../C/CpuArch.h"
+#include "../Common/ItemNameUtils.h"
+
+#include "NsisHandler.h"
 
 #define Get32(p) GetUi32(p)
 
@@ -23,20 +23,20 @@ using namespace NWindows;
 namespace NArchive {
 namespace NNsis {
 
-static const wchar_t *kBcjMethod = L"BCJ";
-static const wchar_t *kUnknownMethod = L"Unknown";
+static const char *kBcjMethod = "BCJ";
+static const char *kUnknownMethod = "Unknown";
 
-static const wchar_t *kMethods[] =
+static const char *kMethods[] =
 {
-  L"Copy",
-  L"Deflate",
-  L"BZip2",
-  L"LZMA"
+  "Copy",
+  "Deflate",
+  "BZip2",
+  "LZMA"
 };
 
 static const int kNumMethods = sizeof(kMethods) / sizeof(kMethods[0]);
 
-STATPROPSTG kProps[] =
+static STATPROPSTG kProps[] =
 {
   { NULL, kpidPath, VT_BSTR},
   { NULL, kpidSize, VT_UI8},
@@ -46,7 +46,7 @@ STATPROPSTG kProps[] =
   { NULL, kpidSolid, VT_BOOL}
 };
 
-STATPROPSTG kArcProps[] =
+static STATPROPSTG kArcProps[] =
 {
   { NULL, kpidMethod, VT_BSTR},
   { NULL, kpidSolid, VT_BOOL}
@@ -116,50 +116,45 @@ STDMETHODIMP CHandler::GetNumberOfItems(UInt32 *numItems)
   return S_OK;
 }
 
-static UString ConvertUInt32ToString(UInt32 value)
+static AString UInt32ToString(UInt32 value)
 {
-  wchar_t buffer[32];
-  ConvertUInt64ToString(value, buffer);
+  char buffer[16];
+  ConvertUInt32ToString(value, buffer);
   return buffer;
 }
 
-static UString GetStringForSizeValue(UInt32 value)
+static AString GetStringForSizeValue(UInt32 value)
 {
   for (int i = 31; i >= 0; i--)
-    if ((UInt32(1) << i) == value)
-      return ConvertUInt32ToString(i);
-  UString result;
+    if (((UInt32)1 << i) == value)
+      return UInt32ToString(i);
+  char c = 'b';
   if (value % (1 << 20) == 0)
   {
-    result += ConvertUInt32ToString(value >> 20);
-    result += L"m";
+    value >>= 20;
+    c = 'm';
   }
   else if (value % (1 << 10) == 0)
   {
-    result += ConvertUInt32ToString(value >> 10);
-    result += L"k";
+    value >>= 10;
+    c = 'k';
   }
-  else
-  {
-    result += ConvertUInt32ToString(value);
-    result += L"b";
-  }
-  return result;
+  return UInt32ToString(value) + c;
 }
 
-UString CHandler::GetMethod(bool useItemFilter, UInt32 dictionary) const
+AString CHandler::GetMethod(bool useItemFilter, UInt32 dictionary) const
 {
   NMethodType::EEnum methodIndex = _archive.Method;
-  UString method;
+  AString method;
   if (_archive.IsSolid && _archive.UseFilter || !_archive.IsSolid && useItemFilter)
   {
     method += kBcjMethod;
-    method += L" ";
+    method += ' ';
   }
   method += (methodIndex < kNumMethods) ? kMethods[methodIndex] : kUnknownMethod;
   if (methodIndex == NMethodType::kLZMA)
   {
-    method += L":";
+    method += ':';
     method += GetStringForSizeValue(_archive.IsSolid ? _archive.DictionarySize: dictionary);
   }
   return method;
@@ -228,12 +223,7 @@ STDMETHODIMP CHandler::GetProperty(UInt32 index, PROPID propID, PROPVARIANT *val
     {
       case kpidPath:
       {
-        UString s;
-        if (_archive.IsUnicode)
-          s = item.GetReducedNameU();
-        else
-          s = MultiByteToUnicodeString(item.GetReducedNameA(), CP_ACP);
-        s = NItemName::WinNameToOSName(s);
+        UString s = NItemName::WinNameToOSName(item.GetReducedName(_archive.IsUnicode));
         if (!s.IsEmpty())
           prop = (const wchar_t *)s;
         break;
@@ -268,12 +258,11 @@ STDMETHODIMP CHandler::GetProperty(UInt32 index, PROPID propID, PROPVARIANT *val
   COM_TRY_END
 }
 
-STDMETHODIMP CHandler::Extract(const UInt32* indices, UInt32 numItems,
-    Int32 _aTestMode, IArchiveExtractCallback *extractCallback)
+STDMETHODIMP CHandler::Extract(const UInt32 *indices, UInt32 numItems,
+    Int32 testMode, IArchiveExtractCallback *extractCallback)
 {
   COM_TRY_BEGIN
-  bool testMode = (_aTestMode != 0);
-  bool allFilesMode = (numItems == UInt32(-1));
+  bool allFilesMode = (numItems == (UInt32)-1);
   if (allFilesMode)
     GetNumberOfItems(&numItems);
   if(numItems == 0)
@@ -320,14 +309,20 @@ STDMETHODIMP CHandler::Extract(const UInt32* indices, UInt32 numItems,
         _inStream, _archive.Method, _archive.FilterFlag, useFilter));
   }
 
+  CByteBuffer byteBuf;
+  const UInt32 kBufferLength = 1 << 16;
+  byteBuf.SetCapacity(kBufferLength);
+  Byte *buffer = byteBuf;
+
   bool dataError = false;
   for (i = 0; i < numItems; i++, currentTotalSize += currentItemSize)
   {
     currentItemSize = 0;
     RINOK(extractCallback->SetCompleted(&currentTotalSize));
     CMyComPtr<ISequentialOutStream> realOutStream;
-    Int32 askMode;
-    askMode = testMode ? NArchive::NExtract::NAskMode::kTest : NArchive::NExtract::NAskMode::kExtract;
+    Int32 askMode = testMode ?
+        NExtract::NAskMode::kTest :
+        NExtract::NAskMode::kExtract;
     UInt32 index = allFilesMode ? i : indices[i];
 
     RINOK(extractCallback->GetStream(index, &realOutStream, askMode));
@@ -336,7 +331,7 @@ STDMETHODIMP CHandler::Extract(const UInt32* indices, UInt32 numItems,
     if (index >= (UInt32)_archive.Items.Size())
     {
       currentItemSize = _archive.Script.Length();
-      if(!testMode && (!realOutStream))
+      if(!testMode && !realOutStream)
         continue;
       RINOK(extractCallback->PrepareOperation(askMode));
       if (!testMode)
@@ -352,7 +347,7 @@ STDMETHODIMP CHandler::Extract(const UInt32* indices, UInt32 numItems,
       else
         GetCompressedSize(index, currentItemSize);
       
-      if(!testMode && (!realOutStream))
+      if(!testMode && !realOutStream)
         continue;
       
       RINOK(extractCallback->PrepareOperation(askMode));
@@ -363,9 +358,6 @@ STDMETHODIMP CHandler::Extract(const UInt32* indices, UInt32 numItems,
         bool sizeIsKnown = false;
         UInt32 fullSize = 0;
 
-        const UInt32 kBufferLength = 1 << 11;
-        Byte buffer[kBufferLength];
-        
         if (_archive.IsSolid)
         {
           UInt64 pos = _archive.GetPosOfSolidItem(index);
@@ -389,12 +381,13 @@ STDMETHODIMP CHandler::Extract(const UInt32* indices, UInt32 numItems,
           }
           if (streamPos == pos)
           {
+            Byte buffer2[4];
             size_t processedSize = 4;
-            RINOK(_archive.Decoder.Read(buffer, &processedSize));
+            RINOK(_archive.Decoder.Read(buffer2, &processedSize));
             if (processedSize != 4)
               return E_FAIL;
             streamPos += processedSize;
-            fullSize = Get32(buffer);
+            fullSize = Get32(buffer2);
             sizeIsKnown = true;
             needDecompress = true;
           }
@@ -409,7 +402,9 @@ STDMETHODIMP CHandler::Extract(const UInt32* indices, UInt32 numItems,
             RINOK(_archive.Decoder.Init(
                 EXTERNAL_CODECS_VARS
                 _inStream, _archive.Method, _archive.FilterFlag, useFilter));
-            fullSize = Get32(buffer);
+            // fullSize = Get32(buffer); // It's bug !!!
+            // Test it: what is exact fullSize?
+            fullSize =  0xFFFFFFFF;
           }
           else
             fullSize = item.Size;
@@ -477,8 +472,8 @@ STDMETHODIMP CHandler::Extract(const UInt32* indices, UInt32 numItems,
     }
     realOutStream.Release();
     RINOK(extractCallback->SetOperationResult(dataError ?
-        NArchive::NExtract::NOperationResult::kDataError :
-        NArchive::NExtract::NOperationResult::kOK));
+        NExtract::NOperationResult::kDataError :
+        NExtract::NOperationResult::kOK));
   }
   return S_OK;
   COM_TRY_END
