@@ -1,21 +1,22 @@
-// AddCommon.cpp
+// ZipAddCommon.cpp
 
 #include "StdAfx.h"
 
-extern "C"
-{
 #include "../../../../C/7zCrc.h"
-}
 
 #include "Windows/PropVariant.h"
-#include "Windows/Defs.h"
-#include "../../MyVersion.h"
+
 #include "../../ICoder.h"
 #include "../../IPassword.h"
+#include "../../MyVersion.h"
+
 #include "../../Common/CreateCoder.h"
 #include "../../Common/StreamObjects.h"
 #include "../../Common/StreamUtils.h"
-#include "../../Compress/LZMA/LZMAEncoder.h"
+
+#include "../../Compress/LzmaEncoder.h"
+#include "../../Compress/PpmdZip.h"
+
 #include "../Common/InStreamWithCRC.h"
 
 #include "ZipAddCommon.h"
@@ -34,7 +35,7 @@ class CLzmaEncoder:
   public ICompressCoder,
   public CMyUnknownImp
 {
-  NCompress::NLZMA::CEncoder *EncoderSpec;
+  NCompress::NLzma::CEncoder *EncoderSpec;
   CMyComPtr<ICompressCoder> Encoder;
   Byte Header[kLzmaHeaderSize];
 public:
@@ -49,7 +50,7 @@ HRESULT CLzmaEncoder::SetCoderProperties(const PROPID *propIDs, const PROPVARIAN
 {
   if (!Encoder)
   {
-    EncoderSpec = new NCompress::NLZMA::CEncoder;
+    EncoderSpec = new NCompress::NLzma::CEncoder;
     Encoder = EncoderSpec;
   }
   CSequentialOutStreamImp *outStreamSpec = new CSequentialOutStreamImp;
@@ -102,7 +103,7 @@ static HRESULT GetStreamCRC(ISequentialInStream *inStream, UInt32 &resultCRC)
 HRESULT CAddCommon::Compress(
     DECL_EXTERNAL_CODECS_LOC_VARS
     ISequentialInStream *inStream, IOutStream *outStream,
-    ICompressProgressInfo *progress, CCompressingResult &operationResult)
+    ICompressProgressInfo *progress, CCompressingResult &opRes)
 {
   CSequentialInStreamWithCRC *inSecCrcStreamSpec = 0;
   CInStreamWithCRC *inCrcStreamSpec = 0;
@@ -139,13 +140,17 @@ HRESULT CAddCommon::Compress(
   }
   Byte method = 0;
   COutStreamReleaser outStreamReleaser;
+  opRes.ExtractVersion = NFileHeader::NCompressionMethod::kExtractVersion_Default;
   for (int i = 0; i < numTestMethods; i++)
   {
+    opRes.ExtractVersion = NFileHeader::NCompressionMethod::kExtractVersion_Default;
     if (inCrcStreamSpec != 0)
       RINOK(inCrcStreamSpec->Seek(0, STREAM_SEEK_SET, NULL));
     RINOK(outStream->Seek(0, STREAM_SEEK_SET, NULL));
     if (_options.PasswordIsDefined)
     {
+      opRes.ExtractVersion = NFileHeader::NCompressionMethod::kExtractVersion_ZipCrypto;
+
       if (!_cryptoStream)
       {
         _cryptoStreamSpec = new CFilterCoder;
@@ -153,22 +158,26 @@ HRESULT CAddCommon::Compress(
       }
       if (_options.IsAesMode)
       {
-        _cryptoStreamSpec->Filter = _aesFilter = _filterAesSpec = new NCrypto::NWzAES::CEncoder;
-        _filterAesSpec->SetKeyMode(_options.AesKeyMode);
-        RINOK(_filterAesSpec->CryptoSetPassword(
-            (const Byte *)(const char *)_options.Password, _options.Password.Length()));
+        opRes.ExtractVersion = NFileHeader::NCompressionMethod::kExtractVersion_Aes;
+        if (!_cryptoStreamSpec->Filter)
+        {
+          _cryptoStreamSpec->Filter = _filterAesSpec = new NCrypto::NWzAes::CEncoder;
+          _filterAesSpec->SetKeyMode(_options.AesKeyMode);
+          RINOK(_filterAesSpec->CryptoSetPassword((const Byte *)(const char *)_options.Password, _options.Password.Length()));
+        }
         RINOK(_filterAesSpec->WriteHeader(outStream));
       }
       else
       {
-        _cryptoStreamSpec->Filter = _zipCryptoFilter = _filterSpec = new NCrypto::NZip::CEncoder;
-        RINOK(_filterSpec->CryptoSetPassword(
-            (const Byte *)(const char *)_options.Password, _options.Password.Length()));
+        if (!_cryptoStreamSpec->Filter)
+        {
+          _cryptoStreamSpec->Filter = _filterSpec = new NCrypto::NZip::CEncoder;
+          _filterSpec->CryptoSetPassword((const Byte *)(const char *)_options.Password, _options.Password.Length());
+        }
         UInt32 crc = 0;
         RINOK(GetStreamCRC(inStream, crc));
         RINOK(inCrcStreamSpec->Seek(0, STREAM_SEEK_SET, NULL));
-        RINOK(_filterSpec->CryptoSetCRC(crc));
-        RINOK(_filterSpec->WriteHeader(outStream));
+        RINOK(_filterSpec->WriteHeader(outStream, crc));
       }
       RINOK(_cryptoStreamSpec->SetOutStream(outStream));
       outStreamReleaser.FilterCoder = _cryptoStreamSpec;
@@ -190,7 +199,6 @@ HRESULT CAddCommon::Compress(
         else
           outStreamNew = outStream;
         RINOK(_copyCoder->Code(inCrcStream, outStreamNew, NULL, NULL, progress));
-        operationResult.ExtractVersion = NFileHeader::NCompressionMethod::kStoreExtractVersion;
         break;
       }
       default:
@@ -199,11 +207,12 @@ HRESULT CAddCommon::Compress(
         {
           if (method == NFileHeader::NCompressionMethod::kLZMA)
           {
+            _compressExtractVersion = NFileHeader::NCompressionMethod::kExtractVersion_LZMA;
             CLzmaEncoder *_lzmaEncoder = new CLzmaEncoder();
             _compressEncoder = _lzmaEncoder;
             NWindows::NCOM::CPropVariant props[] =
             {
-              #ifdef COMPRESS_MT
+              #ifndef _7ZIP_ST
               _options.NumThreads,
               #endif
               _options.Algo,
@@ -214,7 +223,7 @@ HRESULT CAddCommon::Compress(
             };
             PROPID propIDs[] =
             {
-              #ifdef COMPRESS_MT
+              #ifndef _7ZIP_ST
               NCoderPropID::kNumThreads,
               #endif
               NCoderPropID::kAlgorithm,
@@ -228,6 +237,26 @@ HRESULT CAddCommon::Compress(
               numProps--;
             RINOK(_lzmaEncoder->SetCoderProperties(propIDs, props, numProps));
           }
+          else if (method == NFileHeader::NCompressionMethod::kPPMd)
+          {
+            _compressExtractVersion = NFileHeader::NCompressionMethod::kExtractVersion_PPMd;
+            NCompress::NPpmdZip::CEncoder *encoder = new NCompress::NPpmdZip::CEncoder();
+            _compressEncoder = encoder;
+            NWindows::NCOM::CPropVariant props[] =
+            {
+              _options.Algo,
+              _options.MemSize,
+              _options.Order
+              
+            };
+            PROPID propIDs[] =
+            {
+              NCoderPropID::kAlgorithm,
+              NCoderPropID::kUsedMemorySize,
+              NCoderPropID::kOrder
+            };
+            RINOK(encoder->SetCoderProperties(propIDs, props, sizeof(propIDs) / sizeof(propIDs[0])));
+          }
           else
           {
           CMethodId methodId;
@@ -235,8 +264,12 @@ HRESULT CAddCommon::Compress(
           {
             case NFileHeader::NCompressionMethod::kBZip2:
               methodId = kMethodId_BZip2;
+              _compressExtractVersion = NFileHeader::NCompressionMethod::kExtractVersion_BZip2;
               break;
             default:
+              _compressExtractVersion = ((method == NFileHeader::NCompressionMethod::kDeflated64) ?
+                  NFileHeader::NCompressionMethod::kExtractVersion_Deflate64 :
+                  NFileHeader::NCompressionMethod::kExtractVersion_Deflate);
               methodId = kMethodId_ZipBase + method;
               break;
           }
@@ -279,7 +312,7 @@ HRESULT CAddCommon::Compress(
             {
               _options.DicSize,
               _options.NumPasses
-              #ifdef COMPRESS_MT
+              #ifndef _7ZIP_ST
               , _options.NumThreads
               #endif
             };
@@ -287,7 +320,7 @@ HRESULT CAddCommon::Compress(
             {
               NCoderPropID::kDictionarySize,
               NCoderPropID::kNumPasses
-              #ifdef COMPRESS_MT
+              #ifndef _7ZIP_ST
               , NCoderPropID::kNumThreads
               #endif
             };
@@ -305,41 +338,42 @@ HRESULT CAddCommon::Compress(
           outStreamNew = _cryptoStream;
         else
           outStreamNew = outStream;
+        if (_compressExtractVersion > opRes.ExtractVersion)
+          opRes.ExtractVersion = _compressExtractVersion;
         RINOK(_compressEncoder->Code(inCrcStream, outStreamNew, NULL, NULL, progress));
-        operationResult.ExtractVersion = NFileHeader::NCompressionMethod::kDeflateExtractVersion;
         break;
       }
     }
 
-    RINOK(outStream->Seek(0, STREAM_SEEK_CUR, &operationResult.PackSize));
+    RINOK(outStream->Seek(0, STREAM_SEEK_CUR, &opRes.PackSize));
 
     if (inCrcStreamSpec != 0)
     {
-      operationResult.CRC = inCrcStreamSpec->GetCRC();
-      operationResult.UnpackSize = inCrcStreamSpec->GetSize();
+      opRes.CRC = inCrcStreamSpec->GetCRC();
+      opRes.UnpackSize = inCrcStreamSpec->GetSize();
     }
     else
     {
-      operationResult.CRC = inSecCrcStreamSpec->GetCRC();
-      operationResult.UnpackSize = inSecCrcStreamSpec->GetSize();
+      opRes.CRC = inSecCrcStreamSpec->GetCRC();
+      opRes.UnpackSize = inSecCrcStreamSpec->GetSize();
     }
 
     if (_options.PasswordIsDefined)
     {
-      if (operationResult.PackSize < operationResult.UnpackSize +
+      if (opRes.PackSize < opRes.UnpackSize +
           (_options.IsAesMode ? _filterAesSpec->GetHeaderSize() : NCrypto::NZip::kHeaderSize))
         break;
     }
-    else if (operationResult.PackSize < operationResult.UnpackSize)
+    else if (opRes.PackSize < opRes.UnpackSize)
       break;
   }
   if (_options.IsAesMode)
   {
     RINOK(_filterAesSpec->WriteFooter(outStream));
-    RINOK(outStream->Seek(0, STREAM_SEEK_CUR, &operationResult.PackSize));
+    RINOK(outStream->Seek(0, STREAM_SEEK_CUR, &opRes.PackSize));
   }
-  operationResult.Method = method;
-  return outStream->SetSize(operationResult.PackSize);
+  opRes.Method = method;
+  return outStream->SetSize(opRes.PackSize);
 }
 
 }}
