@@ -112,7 +112,7 @@ STDMETHODIMP CHandler::UpdateItems(ISequentialOutStream *outStream, UInt32 numIt
     ui.NewData = IntToBool(newData);
     ui.IndexInArchive = indexInArchive;
     ui.IndexInClient = i;
-    bool existInArchive = (indexInArchive != UInt32(-1));
+    bool existInArchive = (indexInArchive != (UInt32)-1);
     if (existInArchive && newData)
       if (m_Items[indexInArchive].IsAesEncrypted())
         thereAreAesUpdates = true;
@@ -296,14 +296,15 @@ STDMETHODIMP CHandler::UpdateItems(ISequentialOutStream *outStream, UInt32 numIt
       (mainMethod == NFileHeader::NCompressionMethod::kDeflated64);
   bool isLZMA = (mainMethod == NFileHeader::NCompressionMethod::kLZMA);
   bool isLz = (isLZMA || isDeflate);
-  bool isBZip2 = (mainMethod == NFileHeader::NCompressionMethod::kBZip2);
   options.NumPasses = m_NumPasses;
   options.DicSize = m_DicSize;
   options.NumFastBytes = m_NumFastBytes;
   options.NumMatchFinderCycles = m_NumMatchFinderCycles;
   options.NumMatchFinderCyclesDefined = m_NumMatchFinderCyclesDefined;
   options.Algo = m_Algo;
-  #ifdef COMPRESS_MT
+  options.MemSize = m_MemSize;
+  options.Order = m_Order;
+  #ifndef _7ZIP_ST
   options.NumThreads = _numThreads;
   #endif
   if (isLz)
@@ -342,7 +343,7 @@ STDMETHODIMP CHandler::UpdateItems(ISequentialOutStream *outStream, UInt32 numIt
         options.Algo = (level >= 5 ? kLzAlgoX5 :
                                      kLzAlgoX1);
   }
-  if (isBZip2)
+  if (mainMethod == NFileHeader::NCompressionMethod::kBZip2)
   {
     if (options.NumPasses == 0xFFFFFFFF)
       options.NumPasses = (level >= 9 ? kBZip2NumPassesX9 :
@@ -353,6 +354,21 @@ STDMETHODIMP CHandler::UpdateItems(ISequentialOutStream *outStream, UInt32 numIt
                         (level >= 3 ? kBZip2DicSizeX3 :
                                       kBZip2DicSizeX1));
   }
+  if (mainMethod == NFileHeader::NCompressionMethod::kPPMd)
+  {
+    int level2 = level;
+    if (level2 < 1) level2 = 1;
+    if (level2 > 9) level2 = 9;
+
+    if (options.MemSize == 0xFFFFFFFF)
+      options.MemSize = (1 << (19 + (level2 > 8 ? 8 : level2)));
+
+    if (options.Order == 0xFFFFFFFF)
+      options.Order = 3 + level2;
+
+    if (options.Algo == 0xFFFFFFFF)
+      options.Algo = (level2 >= 7 ? 1 : 0);
+  }
 
   return Update(
       EXTERNAL_CODECS_VARS
@@ -363,7 +379,7 @@ STDMETHODIMP CHandler::UpdateItems(ISequentialOutStream *outStream, UInt32 numIt
 
 STDMETHODIMP CHandler::SetProperties(const wchar_t **names, const PROPVARIANT *values, Int32 numProperties)
 {
-  #ifdef COMPRESS_MT
+  #ifndef _7ZIP_ST
   const UInt32 numProcessors = NSystem::GetNumberOfProcessors();
   _numThreads = numProcessors;
   #endif
@@ -395,6 +411,7 @@ STDMETHODIMP CHandler::SetProperties(const wchar_t **names, const PROPVARIANT *v
         else if (m == L"DEFLATE64") m_MainMethod = NFileHeader::NCompressionMethod::kDeflated64;
         else if (m == L"BZIP2") m_MainMethod = NFileHeader::NCompressionMethod::kBZip2;
         else if (m == L"LZMA") m_MainMethod = NFileHeader::NCompressionMethod::kLZMA;
+        else if (m == L"PPMD") m_MainMethod = NFileHeader::NCompressionMethod::kPPMd;
         else return E_INVALIDARG;
       }
       else if (prop.vt == VT_UI4)
@@ -452,6 +469,18 @@ STDMETHODIMP CHandler::SetProperties(const wchar_t **names, const PROPVARIANT *v
       RINOK(ParsePropDictionaryValue(name.Mid(1), prop, dicSize));
       m_DicSize = dicSize;
     }
+    else if (name.Left(3) == L"MEM")
+    {
+      UInt32 memSize = 1 << 24;
+      RINOK(ParsePropDictionaryValue(name.Mid(3), prop, memSize));
+      m_MemSize = memSize;
+    }
+    else if (name[0] == L'O')
+    {
+      UInt32 order = 8;
+      RINOK(ParsePropValue(name.Mid(1), prop, order));
+      m_Order = order;
+    }
     else if (name.Left(4) == L"PASS")
     {
       UInt32 num = kDeflateNumPassesX9;
@@ -473,7 +502,7 @@ STDMETHODIMP CHandler::SetProperties(const wchar_t **names, const PROPVARIANT *v
     }
     else if (name.Left(2) == L"MT")
     {
-      #ifdef COMPRESS_MT
+      #ifndef _7ZIP_ST
       RINOK(ParseMtProp(name.Mid(2), prop, numProcessors, _numThreads));
       #endif
     }
