@@ -23,6 +23,8 @@ static const UINT kIconTypesResId = 100;
 
 #ifdef _WIN32
 #include "Windows/Registry.h"
+#else
+#include "Common/StringConvert.h"
 #endif
 
 using namespace NWindows;
@@ -43,16 +45,20 @@ static CSysString GetLibraryFolderPrefix()
   #else
   const char *p7zip_home_dir = getenv("P7ZIP_HOME_DIR");
   if (p7zip_home_dir == 0) p7zip_home_dir="./";
+#ifdef _UNICODE
+  return MultiByteToUnicodeString(p7zip_home_dir);
+#else
   return p7zip_home_dir;
+#endif
   #endif
 }
 
 #define kCodecsFolderName TEXT("Codecs")
 #define kFormatsFolderName TEXT("Formats")
-static TCHAR *kMainDll = TEXT("7z.dll");
+static const TCHAR *kMainDll = TEXT("7z.dll");
 
 #ifdef _WIN32
-static LPCTSTR kRegistryPath = TEXT("Software\\7-zip");
+static LPCTSTR kRegistryPath = TEXT("Software") TEXT(STRING_PATH_SEPARATOR) TEXT("7-zip");
 static LPCTSTR kProgramPathValue = TEXT("Path");
 static bool ReadPathFromRegistry(HKEY baseKey, CSysString &path)
 {
@@ -71,22 +77,31 @@ static bool ReadPathFromRegistry(HKEY baseKey, CSysString &path)
 CSysString GetBaseFolderPrefixFromRegistry()
 {
   CSysString moduleFolderPrefix = GetLibraryFolderPrefix();
-  NFind::CFileInfo fileInfo;
-  if (NFind::FindFile(moduleFolderPrefix + kMainDll, fileInfo))
-    if (!fileInfo.IsDirectory())
+#ifdef _UNICODE
+  NFind::CFileInfoW fi;
+#else
+  NFind::CFileInfo fi;
+#endif
+  if (NFind::FindFile(moduleFolderPrefix + kMainDll, fi))
+    if (!fi.IsDir())
       return moduleFolderPrefix;
-  if (NFind::FindFile(moduleFolderPrefix + kCodecsFolderName, fileInfo))
-    if (fileInfo.IsDirectory())
+  if (NFind::FindFile(moduleFolderPrefix + kCodecsFolderName, fi))
+    if (fi.IsDir())
       return moduleFolderPrefix;
-  if (NFind::FindFile(moduleFolderPrefix + kFormatsFolderName, fileInfo))
-    if (fileInfo.IsDirectory())
+  if (NFind::FindFile(moduleFolderPrefix + kFormatsFolderName, fi))
+    if (fi.IsDir())
       return moduleFolderPrefix;
   #ifdef _WIN32
-  CSysString path;
-  if (ReadPathFromRegistry(HKEY_CURRENT_USER, path))
-    return path;
-  if (ReadPathFromRegistry(HKEY_LOCAL_MACHINE, path))
-    return path;
+  if (!NFind::DoesFileExist(moduleFolderPrefix + kMainDll) &&
+      !NFind::DoesDirExist(moduleFolderPrefix + kCodecsFolderName) &&
+      !NFind::DoesDirExist(moduleFolderPrefix + kFormatsFolderName))
+  {
+    CSysString path;
+    if (ReadPathFromRegistry(HKEY_CURRENT_USER, path))
+      return path;
+    if (ReadPathFromRegistry(HKEY_LOCAL_MACHINE, path))
+      return path;
+  }
   #endif
   return moduleFolderPrefix;
 }
@@ -96,9 +111,10 @@ typedef UInt32 (WINAPI *GetNumberOfFormatsFunc)(UInt32 *numFormats);
 typedef UInt32 (WINAPI *GetHandlerPropertyFunc)(PROPID propID, PROPVARIANT *value);
 typedef UInt32 (WINAPI *GetHandlerPropertyFunc2)(UInt32 index, PROPID propID, PROPVARIANT *value);
 typedef UInt32 (WINAPI *CreateObjectFunc)(const GUID *clsID, const GUID *iid, void **outObject);
+typedef UInt32 (WINAPI *SetLargePageModeFunc)();
 
 
-static HRESULT GetCoderClass(GetMethodPropertyFunc getMethodProperty, UInt32 index, 
+static HRESULT GetCoderClass(GetMethodPropertyFunc getMethodProperty, UInt32 index,
     PROPID propId, CLSID &clsId, bool &isAssigned)
 {
   NWindows::NCOM::CPropVariant prop;
@@ -117,12 +133,12 @@ static HRESULT GetCoderClass(GetMethodPropertyFunc getMethodProperty, UInt32 ind
 HRESULT CCodecs::LoadCodecs()
 {
   CCodecLib &lib = Libs.Back();
-  lib.GetMethodProperty = (GetMethodPropertyFunc)lib.Lib.GetProcAddress("GetMethodProperty");
+  lib.GetMethodProperty = (GetMethodPropertyFunc)lib.Lib.GetProc("GetMethodProperty");
   if (lib.GetMethodProperty == NULL)
     return S_OK;
 
   UInt32 numMethods = 1;
-  GetNumberOfMethodsFunc getNumberOfMethodsFunc = (GetNumberOfMethodsFunc)lib.Lib.GetProcAddress("GetNumberOfMethods");
+  GetNumberOfMethodsFunc getNumberOfMethodsFunc = (GetNumberOfMethodsFunc)lib.Lib.GetProc("GetNumberOfMethods");
   if (getNumberOfMethodsFunc != NULL)
   {
     RINOK(getNumberOfMethodsFunc(&numMethods));
@@ -143,8 +159,8 @@ HRESULT CCodecs::LoadCodecs()
 }
 
 static HRESULT ReadProp(
-    GetHandlerPropertyFunc getProp, 
-    GetHandlerPropertyFunc2 getProp2, 
+    GetHandlerPropertyFunc getProp,
+    GetHandlerPropertyFunc2 getProp2,
     UInt32 index, PROPID propID, NCOM::CPropVariant &prop)
 {
   if (getProp2)
@@ -153,8 +169,8 @@ static HRESULT ReadProp(
 }
 
 static HRESULT ReadBoolProp(
-    GetHandlerPropertyFunc getProp, 
-    GetHandlerPropertyFunc2 getProp2, 
+    GetHandlerPropertyFunc getProp,
+    GetHandlerPropertyFunc2 getProp2,
     UInt32 index, PROPID propID, bool &res)
 {
   NCOM::CPropVariant prop;
@@ -167,8 +183,8 @@ static HRESULT ReadBoolProp(
 }
 
 static HRESULT ReadStringProp(
-    GetHandlerPropertyFunc getProp, 
-    GetHandlerPropertyFunc2 getProp2, 
+    GetHandlerPropertyFunc getProp,
+    GetHandlerPropertyFunc2 getProp2,
     UInt32 index, PROPID propID, UString &res)
 {
   NCOM::CPropVariant prop;
@@ -182,13 +198,13 @@ static HRESULT ReadStringProp(
 
 #endif
 
-static const unsigned int kNumArcsMax = 32;
+static const unsigned int kNumArcsMax = 48;
 static unsigned int g_NumArcs = 0;
-static const CArcInfo *g_Arcs[kNumArcsMax]; 
-void RegisterArc(const CArcInfo *arcInfo) 
-{ 
+static const CArcInfo *g_Arcs[kNumArcsMax];
+void RegisterArc(const CArcInfo *arcInfo)
+{
   if (g_NumArcs < kNumArcsMax)
-    g_Arcs[g_NumArcs++] = arcInfo; 
+    g_Arcs[g_NumArcs++] = arcInfo;
 }
 
 static void SplitString(const UString &srcString, UStringVector &destStrings)
@@ -216,10 +232,11 @@ static void SplitString(const UString &srcString, UStringVector &destStrings)
     destStrings.Add(s);
 }
 
-void CArcInfoEx::AddExts(const wchar_t* ext, const wchar_t* addExt)
+void CArcInfoEx::AddExts(const wchar_t *ext, const wchar_t *addExt)
 {
   UStringVector exts, addExts;
-  SplitString(ext, exts);
+  if (ext != 0)
+    SplitString(ext, exts);
   if (addExt != 0)
     SplitString(addExt, addExts);
   for (int i = 0; i < exts.Size(); i++)
@@ -242,19 +259,16 @@ HRESULT CCodecs::LoadFormats()
 {
   const NDLL::CLibrary &lib = Libs.Back().Lib;
   GetHandlerPropertyFunc getProp = 0;
-  GetHandlerPropertyFunc2 getProp2 = (GetHandlerPropertyFunc2)
-      lib.GetProcAddress("GetHandlerProperty2");
+  GetHandlerPropertyFunc2 getProp2 = (GetHandlerPropertyFunc2)lib.GetProc("GetHandlerProperty2");
   if (getProp2 == NULL)
   {
-    getProp = (GetHandlerPropertyFunc)
-        lib.GetProcAddress("GetHandlerProperty");
+    getProp = (GetHandlerPropertyFunc)lib.GetProc("GetHandlerProperty");
     if (getProp == NULL)
       return S_OK;
   }
 
   UInt32 numFormats = 1;
-  GetNumberOfFormatsFunc getNumberOfFormats = (GetNumberOfFormatsFunc)
-    lib.GetProcAddress("GetNumberOfFormats");
+  GetNumberOfFormatsFunc getNumberOfFormats = (GetNumberOfFormatsFunc)lib.GetProc("GetNumberOfFormats");
   if (getNumberOfFormats != NULL)
   {
     RINOK(getNumberOfFormats(&numFormats));
@@ -300,50 +314,66 @@ HRESULT CCodecs::LoadFormats()
 }
 
 #ifdef NEW_FOLDER_INTERFACE
-void CCodecLib::LoadIcons()
+void CCodecIcons::LoadIcons(HMODULE m)
 {
-  UString iconTypes = MyLoadStringW((HMODULE)Lib, kIconTypesResId);
+#ifdef _WIN32
+  UString iconTypes = MyLoadStringW(m, kIconTypesResId);
   UStringVector pairs;
   SplitString(iconTypes, pairs);
   for (int i = 0; i < pairs.Size(); i++)
   {
     const UString &s = pairs[i];
     int pos = s.Find(L':');
-    if (pos < 0)
-      continue;
     CIconPair iconPair;
-    const wchar_t *end;
-    UString num = s.Mid(pos + 1);
-    iconPair.IconIndex = (UInt32)ConvertStringToUInt64(num, &end);
-    if (*end != L'\0')
-      continue;
+    iconPair.IconIndex = -1;
+    if (pos < 0)
+      pos = s.Length();
+    else
+    {
+      UString num = s.Mid(pos + 1);
+      if (!num.IsEmpty())
+      {
+        const wchar_t *end;
+        iconPair.IconIndex = (UInt32)ConvertStringToUInt64(num, &end);
+        if (*end != L'\0')
+          continue;
+      }
+    }
     iconPair.Ext = s.Left(pos);
     IconPairs.Add(iconPair);
   }
+#endif // #ifdef _WIN32
 }
 
-int CCodecLib::FindIconIndex(const UString &ext) const
+bool CCodecIcons::FindIconIndex(const UString &ext, int &iconIndex) const
 {
+#ifdef _WIN32
+  iconIndex = -1;
   for (int i = 0; i < IconPairs.Size(); i++)
   {
     const CIconPair &pair = IconPairs[i];
     if (ext.CompareNoCase(pair.Ext) == 0)
-      return pair.IconIndex;
+    {
+      iconIndex = pair.IconIndex;
+      return true;
+    }
   }
-  return -1;
+#endif // #ifdef _WIN32
+  return false;
 }
 #endif
 
 #ifdef _7ZIP_LARGE_PAGES
-extern "C" 
+extern "C"
 {
-  extern SIZE_T g_LargePageSize;
+  extern size_t g_LargePageSize;
 }
 #endif
 
-HRESULT CCodecs::LoadDll(const CSysString &dllPath)
+HRESULT CCodecs::LoadDll(const CSysString &dllPath, bool needCheckDll)
 {
 #ifdef _WIN32
+  if (needCheckDll)
   {
     NDLL::CLibrary library;
     if (!library.LoadEx(dllPath, LOAD_LIBRARY_AS_DATAFILE))
@@ -366,13 +396,13 @@ HRESULT CCodecs::LoadDll(const CSysString &dllPath)
     #ifdef _7ZIP_LARGE_PAGES
     if (g_LargePageSize != 0)
     {
-      SetLargePageModeFunc setLargePageMode = (SetLargePageModeFunc)lib.Lib.GetProcAddress("SetLargePageMode");
+      SetLargePageModeFunc setLargePageMode = (SetLargePageModeFunc)lib.Lib.GetProc("SetLargePageMode");
       if (setLargePageMode != 0)
         setLargePageMode();
     }
     #endif
 
-    lib.CreateObject = (CreateObjectFunc)lib.Lib.GetProcAddress("CreateObject");
+    lib.CreateObject = (CreateObjectFunc)lib.Lib.GetProc("CreateObject");
     if (lib.CreateObject != 0)
     {
       int startSize = Codecs.Size();
@@ -393,13 +423,18 @@ HRESULT CCodecs::LoadDll(const CSysString &dllPath)
 
 HRESULT CCodecs::LoadDllsFromFolder(const CSysString &folderPrefix)
 {
+#ifdef _UNICODE
+  NFile::NFind::CEnumeratorW enumerator(folderPrefix + CSysString(TEXT("*")));
+  NFile::NFind::CFileInfoW fi;
+#else
   NFile::NFind::CEnumerator enumerator(folderPrefix + CSysString(TEXT("*")));
-  NFile::NFind::CFileInfo fileInfo;
-  while (enumerator.Next(fileInfo))
+  NFile::NFind::CFileInfo fi;
+#endif
+  while (enumerator.Next(fi))
   {
-    if (fileInfo.IsDirectory())
+    if (fi.IsDir())
       continue;
-    RINOK(LoadDll(folderPrefix + fileInfo.Name));
+    RINOK(LoadDll(folderPrefix + fi.Name, true));
   }
   return S_OK;
 }
@@ -416,6 +451,12 @@ static inline void SetBuffer(CByteBuffer &bb, const Byte *data, int size)
 
 HRESULT CCodecs::Load()
 {
+  #ifdef NEW_FOLDER_INTERFACE
+  #ifdef _WIN32
+  InternalIcons.LoadIcons(g_hInstance);
+  #endif
+  #endif
+
   Formats.Clear();
   #ifdef EXTERNAL_CODECS
   Codecs.Clear();
@@ -438,47 +479,75 @@ HRESULT CCodecs::Load()
   }
   #ifdef EXTERNAL_CODECS
   const CSysString baseFolder = GetBaseFolderPrefixFromRegistry();
-  RINOK(LoadDll(baseFolder + kMainDll));
+  RINOK(LoadDll(baseFolder + kMainDll, false));
   RINOK(LoadDllsFromFolder(baseFolder + kCodecsFolderName TEXT(STRING_PATH_SEPARATOR)));
   RINOK(LoadDllsFromFolder(baseFolder + kFormatsFolderName TEXT(STRING_PATH_SEPARATOR)));
   #endif
   return S_OK;
 }
 
-int CCodecs::FindFormatForArchiveName(const UString &archivePath) const
+#ifndef _SFX
+
+int CCodecs::FindFormatForArchiveName(const UString &arcPath) const
 {
-  int slashPos1 = archivePath.ReverseFind(L'\\');
-  int slashPos2 = archivePath.ReverseFind(L'.');
-  int dotPos = archivePath.ReverseFind(L'.');
+  int slashPos1 = arcPath.ReverseFind(WCHAR_PATH_SEPARATOR);
+  int slashPos2 = arcPath.ReverseFind(L'.');
+  int dotPos = arcPath.ReverseFind(L'.');
   if (dotPos < 0 || dotPos < slashPos1 || dotPos < slashPos2)
     return -1;
-  UString ext = archivePath.Mid(dotPos + 1);
+  UString ext = arcPath.Mid(dotPos + 1);
   for (int i = 0; i < Formats.Size(); i++)
   {
     const CArcInfoEx &arc = Formats[i];
     if (!arc.UpdateEnabled)
       continue;
-    // if (arc.FindExtension(ext) >= 0)
-    UString mainExt = arc.GetMainExt();
-    if (!mainExt.IsEmpty() && ext.CompareNoCase(mainExt) == 0)
+    if (arc.FindExtension(ext) >= 0)
       return i;
   }
   return -1;
 }
 
+int CCodecs::FindFormatForExtension(const UString &ext) const
+{
+  if (ext.IsEmpty())
+    return -1;
+  for (int i = 0; i < Formats.Size(); i++)
+    if (Formats[i].FindExtension(ext) >= 0)
+      return i;
+  return -1;
+}
+
 int CCodecs::FindFormatForArchiveType(const UString &arcType) const
 {
   for (int i = 0; i < Formats.Size(); i++)
-  {
-    const CArcInfoEx &arc = Formats[i];
-    if (!arc.UpdateEnabled)
-      continue;
-    if (arc.Name.CompareNoCase(arcType) == 0)
+    if (Formats[i].Name.CompareNoCase(arcType) == 0)
       return i;
-  }
   return -1;
 }
 
+bool CCodecs::FindFormatForArchiveType(const UString &arcType, CIntVector &formatIndices) const
+{
+  formatIndices.Clear();
+  for (int pos = 0; pos < arcType.Length();)
+  {
+    int pos2 = arcType.Find('.', pos);
+    if (pos2 < 0)
+      pos2 = arcType.Length();
+    const UString name = arcType.Mid(pos, pos2 - pos);
+    int index = FindFormatForArchiveType(name);
+    if (index < 0 && name != L"*")
+    {
+      formatIndices.Clear();
+      return false;
+    }
+    formatIndices.Add(index);
+    pos = pos2 + 1;
+  }
+  return true;
+}
+
+#endif
+
 #ifdef EXTERNAL_CODECS
 
 #ifdef EXPORT_CODECS
@@ -490,9 +559,9 @@ STDAPI GetMethodProperty(UInt32 codecIndex, PROPID propID, PROPVARIANT *value);
 
 STDMETHODIMP CCodecs::GetNumberOfMethods(UInt32 *numMethods)
 {
-  *numMethods = 
+  *numMethods =
       #ifdef EXPORT_CODECS
-      g_NumCodecs + 
+      g_NumCodecs +
       #endif
       Codecs.Size();
   return S_OK;
@@ -505,7 +574,7 @@ STDMETHODIMP CCodecs::GetProperty(UInt32 index, PROPID propID, PROPVARIANT *valu
     return GetMethodProperty(index, propID, value);
   #endif
 
-  const CDllCodecInfo &ci = Codecs[index 
+  const CDllCodecInfo &ci = Codecs[index
       #ifdef EXPORT_CODECS
       - g_NumCodecs
       #endif
@@ -534,7 +603,7 @@ STDMETHODIMP CCodecs::CreateDecoder(UInt32 index, const GUID *iid, void **coder)
   if (index < g_NumCodecs)
     return CreateCoder2(false, index, iid, coder);
   #endif
-  const CDllCodecInfo &ci = Codecs[index 
+  const CDllCodecInfo &ci = Codecs[index
       #ifdef EXPORT_CODECS
       - g_NumCodecs
       #endif
@@ -550,7 +619,7 @@ STDMETHODIMP CCodecs::CreateEncoder(UInt32 index, const GUID *iid, void **coder)
   if (index < g_NumCodecs)
     return CreateCoder2(true, index, iid, coder);
   #endif
-  const CDllCodecInfo &ci = Codecs[index 
+  const CDllCodecInfo &ci = Codecs[index
       #ifdef EXPORT_CODECS
       - g_NumCodecs
       #endif
@@ -561,7 +630,7 @@ STDMETHODIMP CCodecs::CreateEncoder(UInt32 index, const GUID *iid, void **coder)
 }
 
 HRESULT CCodecs::CreateCoder(const UString &name, bool encode, CMyComPtr<ICompressCoder> &coder) const
-{ 
+{
   for (int i = 0; i < Codecs.Size(); i++)
   {
     const CDllCodecInfo &codec = Codecs[i];
@@ -588,7 +657,7 @@ int CCodecs::GetCodecLibIndex(UInt32 index)
     return -1;
   #endif
   #ifdef EXTERNAL_CODECS
-  const CDllCodecInfo &ci = Codecs[index 
+  const CDllCodecInfo &ci = Codecs[index
       #ifdef EXPORT_CODECS
       - g_NumCodecs
       #endif
@@ -612,7 +681,7 @@ bool CCodecs::GetCodecEncoderIsAssigned(UInt32 index)
   }
   #endif
   #ifdef EXTERNAL_CODECS
-  const CDllCodecInfo &ci = Codecs[index 
+  const CDllCodecInfo &ci = Codecs[index
       #ifdef EXPORT_CODECS
       - g_NumCodecs
       #endif
