@@ -2,24 +2,23 @@
 
 #include "StdAfx.h"
 
-#include "List.h"
-#include "ConsoleClose.h"
-
-#include "Common/StringConvert.h"
-#include "Common/StdOutStream.h"
 #include "Common/IntToString.h"
 #include "Common/MyCom.h"
+#include "Common/StdOutStream.h"
+#include "Common/StringConvert.h"
 
-#include "Windows/PropVariant.h"
-#include "Windows/Defs.h"
-#include "Windows/PropVariantConversions.h"
+#include "Windows/Error.h"
 #include "Windows/FileDir.h"
+#include "Windows/PropVariant.h"
+#include "Windows/PropVariantConversions.h"
 
 #include "../../Archive/IArchive.h"
 
-#include "../Common/PropIDUtils.h"
 #include "../Common/OpenArchive.h"
+#include "../Common/PropIDUtils.h"
 
+#include "ConsoleClose.h"
+#include "List.h"
 #include "OpenCallbackConsole.h"
 
 using namespace NWindows;
@@ -30,17 +29,17 @@ struct CPropIdToName
   const wchar_t *Name;
 };
 
-static CPropIdToName kPropIdToName[] =  
+static CPropIdToName kPropIdToName[] =
 {
   { kpidPath, L"Path" },
   { kpidName, L"Name" },
-  { kpidIsFolder, L"Folder" }, 
+  { kpidIsDir, L"Folder" },
   { kpidSize, L"Size" },
-  { kpidPackedSize, L"Packed Size" },
-  { kpidAttributes, L"Attributes" },
-  { kpidCreationTime, L"Created" },
-  { kpidLastAccessTime, L"Accessed" },
-  { kpidLastWriteTime, L"Modified" },
+  { kpidPackSize, L"Packed Size" },
+  { kpidAttrib, L"Attributes" },
+  { kpidCTime, L"Created" },
+  { kpidATime, L"Accessed" },
+  { kpidMTime, L"Modified" },
   { kpidSolid, L"Solid" },
   { kpidCommented, L"Commented" },
   { kpidEncrypted, L"Encrypted" },
@@ -59,7 +58,7 @@ static CPropIdToName kPropIdToName[] =
   { kpidComment, L"Comment" },
   { kpidPosition, L"Position" },
   { kpidPrefix, L"Prefix" },
-  { kpidNumSubFolders, L"Folders" },
+  { kpidNumSubDirs, L"Folders" },
   { kpidNumSubFiles, L"Files" },
   { kpidUnpackVer, L"Version" },
   { kpidVolume, L"Volume" },
@@ -67,32 +66,42 @@ static CPropIdToName kPropIdToName[] =
   { kpidOffset, L"Offset" },
   { kpidLinks, L"Links" },
   { kpidNumBlocks, L"Blocks" },
-  { kpidNumVolumes, L"Volumes" }
+  { kpidNumVolumes, L"Volumes" },
+
+  { kpidBit64, L"64-bit" },
+  { kpidBigEndian, L"Big-endian" },
+  { kpidCpu, L"CPU" },
+  { kpidPhySize, L"Physical Size" },
+  { kpidHeadersSize, L"Headers Size" },
+  { kpidChecksum, L"Checksum" },
+  { kpidCharacts, L"Characteristics" },
+  { kpidVa, L"Virtual Address" },
+  { kpidId, L"ID" },
+  { kpidShortName, L"Short Name" },
+  { kpidCreatorApp, L"Creator Application"},
+  { kpidSectorSize, L"Sector Size" },
+  { kpidPosixAttrib, L"Mode" },
+  { kpidLink, L"Link" },
+
+  { kpidTotalSize, L"Total Size" },
+  { kpidFreeSpace, L"Free Space" },
+  { kpidClusterSize, L"Cluster Size" },
+  { kpidVolumeName, L"Label" }
 };
 
-static const char kEmptyAttributeChar = '.';
-static const char kDirectoryAttributeChar = 'D';
-static const char kReadonlyAttributeChar  = 'R';
-static const char kHiddenAttributeChar    = 'H';
-static const char kSystemAttributeChar    = 'S';
-static const char kArchiveAttributeChar   = 'A';
+static const char kEmptyAttribChar = '.';
 
 static const char *kListing = "Listing archive: ";
 static const wchar_t *kFilesMessage = L"files";
 static const wchar_t *kDirsMessage = L"folders";
 
-static void GetAttributesString(DWORD wa, bool directory, char *s)
+static void GetAttribString(DWORD wa, bool isDir, char *s)
 {
-  s[0] = ((wa & FILE_ATTRIBUTE_DIRECTORY) != 0 || directory) ? 
-      kDirectoryAttributeChar: kEmptyAttributeChar;
-  s[1] = ((wa & FILE_ATTRIBUTE_READONLY) != 0)? 
-      kReadonlyAttributeChar: kEmptyAttributeChar;
-  s[2] = ((wa & FILE_ATTRIBUTE_HIDDEN) != 0) ? 
-      kHiddenAttributeChar: kEmptyAttributeChar;
-  s[3] = ((wa & FILE_ATTRIBUTE_SYSTEM) != 0) ? 
-      kSystemAttributeChar: kEmptyAttributeChar;
-  s[4] = ((wa & FILE_ATTRIBUTE_ARCHIVE) != 0) ? 
-      kArchiveAttributeChar: kEmptyAttributeChar;
+  s[0] = ((wa & FILE_ATTRIBUTE_DIRECTORY) != 0 || isDir) ? 'D' : kEmptyAttribChar;
+  s[1] = ((wa & FILE_ATTRIBUTE_READONLY) != 0) ? 'R': kEmptyAttribChar;
+  s[2] = ((wa & FILE_ATTRIBUTE_HIDDEN) != 0) ? 'H': kEmptyAttribChar;
+  s[3] = ((wa & FILE_ATTRIBUTE_SYSTEM) != 0) ? 'S': kEmptyAttribChar;
+  s[4] = ((wa & FILE_ATTRIBUTE_ARCHIVE) != 0) ? 'A': kEmptyAttribChar;
   s[5] = '\0';
 }
 
@@ -123,22 +132,22 @@ struct CFieldInfoInit
   int Width;
 };
 
-CFieldInfoInit kStandardFieldTable[] = 
+static CFieldInfoInit kStandardFieldTable[] =
 {
-  { kpidLastWriteTime, L"   Date      Time", kLeft, kLeft, 0, 19 },
-  { kpidAttributes, L"Attr", kRight, kCenter, 1, 5 },
+  { kpidMTime, L"   Date      Time", kLeft, kLeft, 0, 19 },
+  { kpidAttrib, L"Attr", kRight, kCenter, 1, 5 },
   { kpidSize, L"Size", kRight, kRight, 1, 12 },
-  { kpidPackedSize, L"Compressed", kRight, kRight, 1, 12 },
+  { kpidPackSize, L"Compressed", kRight, kRight, 1, 12 },
   { kpidPath, L"Name", kLeft, kLeft, 2, 24 }
 };
 
-void PrintSpaces(int numSpaces)
+static void PrintSpaces(int numSpaces)
 {
   for (int i = 0; i < numSpaces; i++)
     g_StdOut << ' ';
 }
 
-void PrintString(EAdjustment adjustment, int width, const UString &textString)
+static void PrintString(EAdjustment adjustment, int width, const UString &textString)
 {
   const int numSpaces = width - textString.Length();
   int numLeftSpaces = 0;
@@ -168,12 +177,8 @@ public:
   HRESULT Init(IInArchive *archive);
   void PrintTitle();
   void PrintTitleLines();
-  HRESULT PrintItemInfo(IInArchive *archive, 
-      const UString &defaultItemName,
-      const NWindows::NFile::NFind::CFileInfoW &archiveFileInfo,
-      UInt32 index,
-      bool techMode);
-  HRESULT PrintSummaryInfo(UInt64 numFiles, UInt64 numDirs, 
+  HRESULT PrintItemInfo(const CArc &arc, UInt32 index, bool techMode);
+  HRESULT PrintSummaryInfo(UInt64 numFiles, UInt64 numDirs,
       const UInt64 *size, const UInt64 *compressedSize);
 };
 
@@ -204,7 +209,9 @@ static UString GetPropName(PROPID propID, BSTR name)
   }
   if (name)
     return name;
-  return L"?";
+  wchar_t s[16];
+  ConvertUInt32ToString(propID, s);
+  return s;
 }
 
 HRESULT CFieldPrinter::Init(IInArchive *archive)
@@ -232,7 +239,7 @@ void CFieldPrinter::PrintTitle()
   {
     const CFieldInfo &fieldInfo = _fields[i];
     PrintSpaces(fieldInfo.PrefixSpacesWidth);
-    PrintString(fieldInfo.TitleAdjustment, 
+    PrintString(fieldInfo.TitleAdjustment,
       ((fieldInfo.PropID == kpidPath) ? 0: fieldInfo.Width), fieldInfo.Name);
   }
 }
@@ -249,22 +256,22 @@ void CFieldPrinter::PrintTitleLines()
 }
 
 
-BOOL IsFileTimeZero(CONST FILETIME *lpFileTime)
+static BOOL IsFileTimeZero(CONST FILETIME *lpFileTime)
 {
   return (lpFileTime->dwLowDateTime == 0) && (lpFileTime->dwHighDateTime == 0);
 }
 
 static const char *kEmptyTimeString = "                   ";
-void PrintTime(const NCOM::CPropVariant &propVariant)
+static void PrintTime(const NCOM::CPropVariant &prop)
 {
-  if (propVariant.vt != VT_FILETIME)
+  if (prop.vt != VT_FILETIME)
     throw "incorrect item";
-  if (IsFileTimeZero(&propVariant.filetime))
+  if (IsFileTimeZero(&prop.filetime))
     g_StdOut << kEmptyTimeString;
   else
   {
     FILETIME localFileTime;
-    if (!FileTimeToLocalFileTime(&propVariant.filetime, &localFileTime))
+    if (!FileTimeToLocalFileTime(&prop.filetime, &localFileTime))
       throw "FileTimeToLocalFileTime error";
     char s[32];
     if (ConvertFileTimeToString(localFileTime, s, true, true))
@@ -274,11 +281,7 @@ void PrintTime(const NCOM::CPropVariant &propVariant)
   }
 }
 
-HRESULT CFieldPrinter::PrintItemInfo(IInArchive *archive, 
-    const UString &defaultItemName, 
-    const NWindows::NFile::NFind::CFileInfoW &archiveFileInfo,
-    UInt32 index,
-    bool techMode)
+HRESULT CFieldPrinter::PrintItemInfo(const CArc &arc, UInt32 index, bool techMode)
 {
   /*
   if (techMode)
@@ -294,67 +297,52 @@ HRESULT CFieldPrinter::PrintItemInfo(IInArchive *archive,
     if (!techMode)
       PrintSpaces(fieldInfo.PrefixSpacesWidth);
 
-    NCOM::CPropVariant propVariant;
+    NCOM::CPropVariant prop;
     if (fieldInfo.PropID == kpidPath)
     {
       UString s;
-      RINOK(GetArchiveItemPath(archive, index, defaultItemName, s));
-      propVariant = s;
+      RINOK(arc.GetItemPath(index, s));
+      prop = s;
     }
     else
     {
-      RINOK(archive->GetProperty(index, fieldInfo.PropID, &propVariant));
+      RINOK(arc.Archive->GetProperty(index, fieldInfo.PropID, &prop));
     }
     if (techMode)
     {
       g_StdOut << fieldInfo.Name << " = ";
     }
     int width = (fieldInfo.PropID == kpidPath) ? 0: fieldInfo.Width;
-    if (propVariant.vt == VT_EMPTY)
+    if (fieldInfo.PropID == kpidAttrib && (prop.vt == VT_EMPTY || prop.vt == VT_UI4))
     {
-      switch(fieldInfo.PropID)
-      {
-        case kpidPath:
-          propVariant = defaultItemName;
-          break;
-        case kpidLastWriteTime:
-          propVariant = archiveFileInfo.LastWriteTime;
-          break;
-        default:
-          if (techMode)
-            g_StdOut << endl;
-          else
-            PrintSpaces(width);
-          continue;
-      }
-    }
-    if (fieldInfo.PropID == kpidLastWriteTime)
-    {
-      PrintTime(propVariant);
-    }
-    else if (fieldInfo.PropID == kpidAttributes)
-    {
-      if (propVariant.vt != VT_UI4)
-        throw "incorrect item";
-      UInt32 attributes = propVariant.ulVal;
+      UInt32 attrib = (prop.vt == VT_EMPTY) ? 0 : prop.ulVal;
       bool isFolder;
-      RINOK(IsArchiveItemFolder(archive, index, isFolder));
+      RINOK(IsArchiveItemFolder(arc.Archive, index, isFolder));
       char s[8];
-      GetAttributesString(attributes, isFolder, s);
+      GetAttribString(attrib, isFolder, s);
       g_StdOut << s;
     }
-    else if (propVariant.vt == VT_BSTR)
+    else if (prop.vt == VT_EMPTY)
+    {
+      if (!techMode)
+        PrintSpaces(width);
+    }
+    else if (fieldInfo.PropID == kpidMTime)
+    {
+      PrintTime(prop);
+    }
+    else if (prop.vt == VT_BSTR)
     {
       if (techMode)
-        g_StdOut << propVariant.bstrVal;
+        g_StdOut << prop.bstrVal;
       else
-        PrintString(fieldInfo.TextAdjustment, width, propVariant.bstrVal);
+        PrintString(fieldInfo.TextAdjustment, width, prop.bstrVal);
     }
     else
     {
-      UString s = ConvertPropertyToString(propVariant, fieldInfo.PropID);
-      s.Replace(wchar_t(0xA), L' '); 
-      s.Replace(wchar_t(0xD), L' '); 
+      UString s = ConvertPropertyToString(prop, fieldInfo.PropID);
+      s.Replace(wchar_t(0xA), L' ');
+      s.Replace(wchar_t(0xD), L' ');
 
       if (techMode)
         g_StdOut << s;
@@ -367,7 +355,7 @@ HRESULT CFieldPrinter::PrintItemInfo(IInArchive *archive,
   return S_OK;
 }
 
-void PrintNumberString(EAdjustment adjustment, int width, const UInt64 *value)
+static void PrintNumberString(EAdjustment adjustment, int width, const UInt64 *value)
 {
   wchar_t textString[32] = { 0 };
   if (value != NULL)
@@ -376,17 +364,17 @@ void PrintNumberString(EAdjustment adjustment, int width, const UInt64 *value)
 }
 
 
-HRESULT CFieldPrinter::PrintSummaryInfo(UInt64 numFiles, UInt64 numDirs, 
+HRESULT CFieldPrinter::PrintSummaryInfo(UInt64 numFiles, UInt64 numDirs,
     const UInt64 *size, const UInt64 *compressedSize)
 {
   for (int i = 0; i < _fields.Size(); i++)
   {
     const CFieldInfo &fieldInfo = _fields[i];
     PrintSpaces(fieldInfo.PrefixSpacesWidth);
-    NCOM::CPropVariant propVariant;
+    NCOM::CPropVariant prop;
     if (fieldInfo.PropID == kpidSize)
       PrintNumberString(fieldInfo.TextAdjustment, fieldInfo.Width, size);
-    else if (fieldInfo.PropID == kpidPackedSize)
+    else if (fieldInfo.PropID == kpidPackSize)
       PrintNumberString(fieldInfo.TextAdjustment, fieldInfo.Width, compressedSize);
     else if (fieldInfo.PropID == kpidPath)
     {
@@ -402,7 +390,7 @@ HRESULT CFieldPrinter::PrintSummaryInfo(UInt64 numFiles, UInt64 numDirs,
       temp += kDirsMessage;
       PrintString(fieldInfo.TextAdjustment, 0, temp);
     }
-    else 
+    else
       PrintString(fieldInfo.TextAdjustment, fieldInfo.Width, L"");
   }
   return S_OK;
@@ -410,20 +398,29 @@ HRESULT CFieldPrinter::PrintSummaryInfo(UInt64 numFiles, UInt64 numDirs,
 
 bool GetUInt64Value(IInArchive *archive, UInt32 index, PROPID propID, UInt64 &value)
 {
-  NCOM::CPropVariant propVariant;
-  if (archive->GetProperty(index, propID, &propVariant) != S_OK)
+  NCOM::CPropVariant prop;
+  if (archive->GetProperty(index, propID, &prop) != S_OK)
     throw "GetPropertyValue error";
-  if (propVariant.vt == VT_EMPTY)
+  if (prop.vt == VT_EMPTY)
     return false;
-  value = ConvertPropVariantToUInt64(propVariant);
+  value = ConvertPropVariantToUInt64(prop);
   return true;
 }
 
-HRESULT ListArchives(
-    CCodecs *codecs,
-    UStringVector &archivePaths, UStringVector &archivePathsFull,
+static void PrintPropPair(const wchar_t *name, const wchar_t *value)
+{
+  g_StdOut << name << " = " << value << endl;
+}
+
+HRESULT ListArchives(CCodecs *codecs, const CIntVector &formatIndices,
+    bool stdInMode,
+    UStringVector &arcPaths, UStringVector &arcPathsFull,
     const NWildcard::CCensorNode &wildcardCensor,
-    bool enableHeaders, bool techMode, bool &passwordEnabled, UString &password, UInt64 &numErrors)
+    bool enableHeaders, bool techMode,
+    #ifndef _NO_CRYPTO
+    bool &passwordEnabled, UString &password,
+    #endif
+    UInt64 &numErrors)
 {
   numErrors = 0;
   CFieldPrinter fieldPrinter;
@@ -432,77 +429,129 @@ HRESULT ListArchives(
 
   UInt64 numFiles2 = 0, numDirs2 = 0, totalPackSize2 = 0, totalUnPackSize2 = 0;
   UInt64 *totalPackSizePointer2 = 0, *totalUnPackSizePointer2 = 0;
-  for (int i = 0; i < archivePaths.Size(); i++)
+  int numArcs = /* stdInMode ? 1 : */ arcPaths.Size();
+  for (int i = 0; i < numArcs; i++)
   {
-    const UString &archiveName = archivePaths[i];
-    NFile::NFind::CFileInfoW archiveFileInfo;
-    if (!NFile::NFind::FindFile(archiveName, archiveFileInfo) || archiveFileInfo.IsDirectory())
+    const UString &archiveName = arcPaths[i];
+    UInt64 arcPackSize = 0;
+    if (!stdInMode)
     {
-      g_StdOut << endl << "Error: " << archiveName << " is not archive" << endl;
-      numErrors++;
-      continue;
-    }
-    if (archiveFileInfo.IsDirectory())
-    {
-      g_StdOut << endl << "Error: " << archiveName << " is not file" << endl;
-      numErrors++;
-      continue;
+      NFile::NFind::CFileInfoW fi;
+      if (!fi.Find(archiveName) || fi.IsDir())
+      {
+        g_StdOut << endl << "Error: " << archiveName << " is not file" << endl;
+        numErrors++;
+        continue;
+      }
+      arcPackSize = fi.Size;
     }
 
     CArchiveLink archiveLink;
 
     COpenCallbackConsole openCallback;
     openCallback.OutStream = &g_StdOut;
+
+    #ifndef _NO_CRYPTO
+
     openCallback.PasswordIsDefined = passwordEnabled;
     openCallback.Password = password;
 
-    HRESULT result = MyOpenArchive(codecs, archiveName, archiveLink, &openCallback);
+    #endif
+
+    HRESULT result = archiveLink.Open2(codecs, formatIndices, stdInMode, NULL, archiveName, &openCallback);
     if (result != S_OK)
     {
-      g_StdOut << endl << "Error: " << archiveName << " is not supported archive" << endl;
+      if (result == E_ABORT)
+        return result;
+      g_StdOut << endl << "Error: " << archiveName << ": ";
+      if (result == S_FALSE)
+      {
+        #ifndef _NO_CRYPTO
+        if (openCallback.Open_WasPasswordAsked())
+          g_StdOut << "Can not open encrypted archive. Wrong password?";
+        else
+        #endif
+          g_StdOut << "Can not open file as archive";
+      }
+      else if (result == E_OUTOFMEMORY)
+        g_StdOut << "Can't allocate required memory";
+      else
+        g_StdOut << NError::MyFormatMessage(result);
+      g_StdOut << endl;
       numErrors++;
       continue;
     }
 
+    if (!stdInMode)
     for (int v = 0; v < archiveLink.VolumePaths.Size(); v++)
     {
-      int index = archivePathsFull.FindInSorted(archiveLink.VolumePaths[v]);
+      int index = arcPathsFull.FindInSorted(archiveLink.VolumePaths[v]);
       if (index >= 0 && index > i)
       {
-        archivePaths.Delete(index);
-        archivePathsFull.Delete(index);
+        arcPaths.Delete(index);
+        arcPathsFull.Delete(index);
+        numArcs = arcPaths.Size();
       }
     }
 
-    IInArchive *archive = archiveLink.GetArchive();
-    const UString defaultItemName = archiveLink.GetDefaultItemName();
-
     if (enableHeaders)
     {
       g_StdOut << endl << kListing << archiveName << endl << endl;
 
-      UInt32 numProps;
-      if (archive->GetNumberOfArchiveProperties(&numProps) == S_OK)
+      for (int i = 0; i < archiveLink.Arcs.Size(); i++)
       {
-        for (UInt32 i = 0; i < numProps; i++)
+        const CArc &arc = archiveLink.Arcs[i];
+        
+        g_StdOut << "--\n";
+        PrintPropPair(L"Path", arc.Path);
+        PrintPropPair(L"Type", codecs->Formats[arc.FormatIndex].Name);
+        UInt32 numProps;
+        IInArchive *archive = arc.Archive;
+        if (archive->GetNumberOfArchiveProperties(&numProps) == S_OK)
         {
-          CMyComBSTR name;
-          PROPID propID;
-          VARTYPE vt;
-          if (archive->GetArchivePropertyInfo(i, &name, &propID, &vt) != S_OK)
-            continue;
-          NCOM::CPropVariant prop;
-          if (archive->GetArchiveProperty(propID, &prop) != S_OK)
-            continue;
-          UString s = ConvertPropertyToString(prop, propID);
-          if (!s.IsEmpty())
-            g_StdOut << GetPropName(propID, name) << " = " << s << endl;
+          for (UInt32 j = 0; j < numProps; j++)
+          {
+            CMyComBSTR name;
+            PROPID propID;
+            VARTYPE vt;
+            if (archive->GetArchivePropertyInfo(j, &name, &propID, &vt) != S_OK)
+              continue;
+            NCOM::CPropVariant prop;
+            if (archive->GetArchiveProperty(propID, &prop) != S_OK)
+              continue;
+            UString s = ConvertPropertyToString(prop, propID);
+            if (!s.IsEmpty())
+              PrintPropPair(GetPropName(propID, name), s);
+          }
         }
+        if (i != archiveLink.Arcs.Size() - 1)
+        {
+          UInt32 numProps;
+          g_StdOut << "----\n";
+          if (archive->GetNumberOfProperties(&numProps) == S_OK)
+          {
+            UInt32 mainIndex = archiveLink.Arcs[i + 1].SubfileIndex;
+            for (UInt32 j = 0; j < numProps; j++)
+            {
+              CMyComBSTR name;
+              PROPID propID;
+              VARTYPE vt;
+              if (archive->GetPropertyInfo(j, &name, &propID, &vt) != S_OK)
+                continue;
+              NCOM::CPropVariant prop;
+              if (archive->GetProperty(mainIndex, propID, &prop) != S_OK)
+                continue;
+              UString s = ConvertPropertyToString(prop, propID);
+              if (!s.IsEmpty())
+                PrintPropPair(GetPropName(propID, name), s);
+            }
+          }
+        }
+        
       }
+      g_StdOut << endl;
       if (techMode)
         g_StdOut << "----------\n";
-      if (numProps > 0)
-        g_StdOut << endl;
     }
 
     if (enableHeaders && !techMode)
@@ -513,6 +562,8 @@ HRESULT ListArchives(
       g_StdOut << endl;
     }
 
+    const CArc &arc = archiveLink.Arcs.Back();
+    IInArchive *archive = arc.Archive;
     if (techMode)
     {
       RINOK(fieldPrinter.Init(archive));
@@ -521,27 +572,30 @@ HRESULT ListArchives(
     UInt64 *totalPackSizePointer = 0, *totalUnPackSizePointer = 0;
     UInt32 numItems;
     RINOK(archive->GetNumberOfItems(&numItems));
-    for(UInt32 i = 0; i < numItems; i++)
+    for (UInt32 i = 0; i < numItems; i++)
     {
       if (NConsoleClose::TestBreakSignal())
         return E_ABORT;
 
       UString filePath;
-      RINOK(GetArchiveItemPath(archive, i, defaultItemName, filePath));
+      HRESULT res = arc.GetItemPath(i, filePath);
+      if (stdInMode && res == E_INVALIDARG)
+        break;
+      RINOK(res);
 
       bool isFolder;
       RINOK(IsArchiveItemFolder(archive, i, isFolder));
       if (!wildcardCensor.CheckPath(filePath, !isFolder))
         continue;
       
-      fieldPrinter.PrintItemInfo(archive, defaultItemName, archiveFileInfo, i, techMode);
+      fieldPrinter.PrintItemInfo(arc, i, techMode);
       
       UInt64 packSize, unpackSize;
       if (!GetUInt64Value(archive, i, kpidSize, unpackSize))
         unpackSize = 0;
       else
         totalUnPackSizePointer = &totalUnPackSize;
-      if (!GetUInt64Value(archive, i, kpidPackedSize, packSize))
+      if (!GetUInt64Value(archive, i, kpidPackSize, packSize))
         packSize = 0;
       else
         totalPackSizePointer = &totalPackSize;
@@ -555,6 +609,19 @@ HRESULT ListArchives(
       totalPackSize += packSize;
       totalUnPackSize += unpackSize;
     }
+
+    if (!stdInMode && totalPackSizePointer == 0)
+    {
+      if (archiveLink.VolumePaths.Size() != 0)
+        arcPackSize += archiveLink.VolumesSize;
+      totalPackSize = (numFiles == 0) ? 0 : arcPackSize;
+      totalPackSizePointer = &totalPackSize;
+    }
+    if (totalUnPackSizePointer == 0 && numFiles == 0)
+    {
+      totalUnPackSize = 0;
+      totalUnPackSizePointer = &totalUnPackSize;
+    }
     if (enableHeaders && !techMode)
     {
       fieldPrinter.PrintTitleLines();
@@ -575,14 +642,14 @@ HRESULT ListArchives(
     numFiles2 += numFiles;
     numDirs2 += numDirs;
   }
-  if (enableHeaders && !techMode && archivePaths.Size() > 1)
+  if (enableHeaders && !techMode && numArcs > 1)
   {
     g_StdOut << endl;
     fieldPrinter.PrintTitleLines();
     g_StdOut << endl;
     fieldPrinter.PrintSummaryInfo(numFiles2, numDirs2, totalUnPackSizePointer2, totalPackSizePointer2);
     g_StdOut << endl;
-    g_StdOut << "Archives: " << archivePaths.Size() << endl;
+    g_StdOut << "Archives: " << numArcs << endl;
   }
   return S_OK;
 }
